@@ -322,8 +322,8 @@ static const short yyrline[] = { 0,
    924,   927,   930,   933,   936,   939,   940,   944,   945,   948,
    951,   954,   955,   959,   963,   964,   968,   973,   978,   984,
    988,   995,   998,  1006,  1018,  1034,  1044,  1080,  1096,  1119,
-  1135,  1136,  1140,  1177,  1178,  1184,  1185,  1194,  1202,  1205,
-  1208,  1211,  1214,  1217,  1218,  1221,  1224,  1225,  1233,  1240
+  1136,  1137,  1141,  1178,  1179,  1185,  1186,  1195,  1203,  1206,
+  1209,  1212,  1215,  1218,  1219,  1222,  1225,  1226,  1234,  1241
 };
 #endif
 
@@ -1572,7 +1572,7 @@ case 89:
 {
 	yyval.tree = IDL_op_dcl_new (yyvsp[-6].boolean, yyvsp[-5].tree, yyvsp[-4].tree, yyvsp[-3].treedata.tree, yyvsp[-1].tree, yyvsp[0].tree);
 	IDL_OP_DCL (yyval.tree).f_noscript = yyvsp[-7].boolean;
-	IDL_OP_DCL (yyval.tree).f_varargs = (gboolean) yyvsp[-3].treedata.data;
+	IDL_OP_DCL (yyval.tree).f_varargs = (gboolean) GPOINTER_TO_INT (yyvsp[-3].treedata.data);
 ;
     break;}
 case 91:
@@ -1599,14 +1599,14 @@ case 96:
 #line 653 "parser.y"
 {
 	yyval.treedata.tree = yyvsp[-2].tree;
-	yyval.treedata.data = (gpointer) yyvsp[-1].boolean;
+	yyval.treedata.data = GINT_TO_POINTER (yyvsp[-1].boolean);
 ;
     break;}
 case 97:
 #line 657 "parser.y"
 {
 	yyval.treedata.tree = NULL;
-	yyval.treedata.data = (gpointer) yyvsp[-1].boolean;
+	yyval.treedata.data = GINT_TO_POINTER (yyvsp[-1].boolean);
 ;
     break;}
 case 98:
@@ -2035,10 +2035,10 @@ case 227:
 		     ++i)
 			if (IDL_NODE_UP (q))
 				q = IDL_NODE_UP (q);
-		
+
 		if (q) {
-			do_token_error (q, "Duplicate identifier conflicts with", FALSE);
-			IDL_tree_error (q, "Previous declaration");
+			IDL_tree_error (yyvsp[0].tree, "`%s' conflicts", IDL_IDENT (yyvsp[0].tree).str);
+			do_token_error (q, "with", FALSE);
 		} else
 			yyerrorv ("`%s' duplicate identifier", IDL_IDENT (yyvsp[0].tree).str);
 
@@ -2100,7 +2100,8 @@ case 230:
 {
 	IDL_tree p;
 
-	if ((p = IDL_ns_lookup_this_scope (__IDL_root_ns, IDL_NS (__IDL_root_ns).file, yyvsp[0].tree, NULL)) == NULL) {
+	if ((p = IDL_ns_lookup_this_scope (
+		__IDL_root_ns,IDL_NS (__IDL_root_ns).file, yyvsp[0].tree, NULL)) == NULL) {
 		yyerrorv ("`%s' undeclared identifier", IDL_IDENT (yyvsp[0].tree).str);
 		IDL_tree_free (yyvsp[0].tree);
 		YYABORT;
@@ -2113,15 +2114,15 @@ case 230:
 ;
     break;}
 case 231:
-#line 1135 "parser.y"
+#line 1136 "parser.y"
 { yyval.tree = list_start (yyvsp[0].tree, TRUE); ;
     break;}
 case 232:
-#line 1137 "parser.y"
+#line 1138 "parser.y"
 { yyval.tree = list_chain (yyvsp[-2].tree, yyvsp[0].tree, TRUE); ;
     break;}
 case 233:
-#line 1140 "parser.y"
+#line 1141 "parser.y"
 {
 	IDL_tree literal, ident = NULL;
 	IDL_longlong_t value = 0;
@@ -2159,22 +2160,22 @@ case 233:
 ;
     break;}
 case 234:
-#line 1177 "parser.y"
+#line 1178 "parser.y"
 { yyval.declspec = 0; ;
     break;}
 case 235:
-#line 1178 "parser.y"
+#line 1179 "parser.y"
 {
 	yyval.declspec = IDL_parse_declspec (yyvsp[0].str);
 	free (yyvsp[0].str);
 ;
     break;}
 case 236:
-#line 1184 "parser.y"
+#line 1185 "parser.y"
 { yyval.hash_table = NULL; ;
     break;}
 case 237:
-#line 1185 "parser.y"
+#line 1186 "parser.y"
 {
 	/* Enable property scanning */
 	if (__IDL_flags & IDLF_XPIDL)
@@ -2186,7 +2187,7 @@ case 237:
 ;
     break;}
 case 238:
-#line 1194 "parser.y"
+#line 1195 "parser.y"
 {
 	/* Disable property scanning */
 	if (__IDL_flags & IDLF_XPIDL)
@@ -2195,35 +2196,35 @@ case 238:
 ;
     break;}
 case 239:
-#line 1202 "parser.y"
+#line 1203 "parser.y"
 { yyval.tree = IDL_integer_new (yyvsp[0].integer); ;
     break;}
 case 240:
-#line 1205 "parser.y"
+#line 1206 "parser.y"
 { yyval.tree = IDL_string_new (yyvsp[0].str); ;
     break;}
 case 241:
-#line 1208 "parser.y"
+#line 1209 "parser.y"
 { yyval.tree = IDL_char_new (yyvsp[0].str); ;
     break;}
 case 242:
-#line 1211 "parser.y"
+#line 1212 "parser.y"
 { yyval.tree = IDL_fixed_new (yyvsp[0].str); ;
     break;}
 case 243:
-#line 1214 "parser.y"
+#line 1215 "parser.y"
 { yyval.tree = IDL_float_new (yyvsp[0].floatp); ;
     break;}
 case 244:
-#line 1217 "parser.y"
+#line 1218 "parser.y"
 { yyval.tree = IDL_boolean_new (TRUE); ;
     break;}
 case 245:
-#line 1218 "parser.y"
+#line 1219 "parser.y"
 { yyval.tree = IDL_boolean_new (FALSE); ;
     break;}
 case 248:
-#line 1225 "parser.y"
+#line 1226 "parser.y"
 {
 	char *catstr = (char *) malloc (strlen (yyvsp[-1].str) + strlen (yyvsp[0].str) + 1);
 	strcpy (catstr, yyvsp[-1].str); free (yyvsp[-1].str);
@@ -2232,7 +2233,7 @@ case 248:
 ;
     break;}
 case 249:
-#line 1233 "parser.y"
+#line 1234 "parser.y"
 {
 	char *s = IDL_do_escapes (yyvsp[0].str);
 	free (yyvsp[0].str);
@@ -2240,7 +2241,7 @@ case 249:
 ;
     break;}
 case 250:
-#line 1240 "parser.y"
+#line 1241 "parser.y"
 {
 	char *s = IDL_do_escapes (yyvsp[0].str);
 	free (yyvsp[0].str);
@@ -2445,7 +2446,7 @@ yyerrhandle:
   yystate = yyn;
   goto yynewstate;
 }
-#line 1247 "parser.y"
+#line 1248 "parser.y"
 
 
 static const char *IDL_ns_get_cur_prefix (IDL_ns ns)
@@ -2767,13 +2768,9 @@ static int do_token_error (IDL_tree p, const char *message, gboolean prev)
 	assert (what != NULL);
 	
 	if (who && *who)
-		yyerrorlv ("%s %s `%s'",
-			   prev ? __IDL_prev_token_line - __IDL_cur_token_line : 0,
-			   message, what, who);
+		IDL_tree_error (p, "%s %s `%s'", message, what, who);
 	else
-		yyerrorlv ("%s %s",
-			   prev ? __IDL_prev_token_line - __IDL_cur_token_line : 0,
-			   message, what);
+		IDL_tree_error (p, "%s %s", message, what);
 	
 	return dienow;
 }
