@@ -1,5 +1,12 @@
 #include "config.h"
+#ifndef _XOPEN_SOURCE_EXTENDED
+#   define _XOPEN_SOURCE_EXTENDED
+#   define WE_DEFINED_XOPEN_SOURCE_EXTENDED
+#endif
 #include "iiop-endianP.h"
+#ifdef WE_DEFINED_XOPEN_SOURCE_EXTENDED
+#   undef _XOPEN_SOURCE_EXTENDED
+#endif
 #include "IIOP.h"
 #include "IIOP-private.h"
 #include "giop-msg-buffer.h"
@@ -10,14 +17,30 @@
 #include <fcntl.h>
 #include <sys/socket.h>
 #include <sys/un.h>
+#ifndef _XOPEN_SOURCE_EXTENDED
+#   define _XOPEN_SOURCE_EXTENDED
+#endif
 #include <arpa/inet.h>
 #include <netdb.h>
+#ifdef WE_DEFINED_XOPEN_SOURCE_EXTENDED
+#   undef _XOPEN_SOURCE_EXTENDED
+#endif
 #include <ctype.h>
 #include <string.h>
 #include <sys/time.h>
 #include <sys/ioctl.h>
 #include <signal.h>
 
+#if 0
+#include <malloc.h>
+
+static struct mallinfo mi1, mi2;
+
+#define AM() mi1 = mallinfo();
+#define PM(x) mi2 = mallinfo(); printf(x ": used %d, now %d\n", \
+mi2.uordblks - mi1.uordblks, mi2.uordblks);
+#endif
+
 #if defined(HAVE_POLL) && defined(I_WANT_POLL)
 #define USE_POLL
 #else
@@ -43,14 +66,13 @@ static void giop_connection_remove_from_list    (GIOPConnection *cnx);
 
 static void iiop_init                           (void);
 static void iiop_connection_destroy   		(IIOPConnection *connection);
-static int  iiop_get_fd               		(GIOPConnection *giop_connection);
 static IIOPConnection *iiop_connection_new      (const char *host, gushort port);
 static IIOPConnection *iiop_connection_unix_new (const char *sockpath);
 static void iiop_unlink_unix_sockets            (void);
 
 DEFINE_LOCK(giop_connection_list);
 GIOPConnectionList giop_connection_list;
-static GSList *iiop_unix_socket_list;
+static GSList *iiop_unix_socket_list = NULL;
 
 /*
  * giop_init
@@ -150,11 +172,19 @@ void giop_connection_free(GIOPConnection *connection)
 
   giop_connection_remove_from_list(connection);
 
-  if(connection->destroy_func)
+  if(connection->is_valid && connection->destroy_func)
     connection->destroy_func(connection);
 
   connection->is_valid = FALSE;
 
+  if(connection->incoming_msg) {
+    GIOPRecvBuffer *buf;
+
+    buf = connection->incoming_msg;
+    connection->incoming_msg = NULL;
+    giop_recv_buffer_unuse(buf);
+  }
+
   g_free(connection);
 }
 
@@ -304,10 +334,7 @@ giop_connection_remove_from_list(GIOPConnection *cnx)
 						 link);
   g_list_free_1(link);
 
-  if(cnx->is_valid) {
-    giop_connection_unref(cnx);
-    cnx->is_valid = FALSE;
-  }
+  giop_connection_unref(cnx);
 
   giop_connection_list_recreate();
  out:
@@ -334,7 +361,6 @@ giop_connection_remove_from_list(GIOPConnection *cnx)
 static void
 iiop_init(void)
 {
-  iiop_unix_socket_list = NULL;
   g_atexit(iiop_unlink_unix_sockets);
 }
 
@@ -363,9 +389,6 @@ iiop_connection_init(IIOPConnection *connection,
   GIOP_CONNECTION(connection)->destroy_func =
     (gpointer)iiop_connection_destroy;
 
-  GIOP_CONNECTION(connection)->get_fd_func =
-    (gpointer)iiop_get_fd;
-
   connection->icnxtype = iioptype;
 }
 
@@ -393,12 +416,12 @@ iiop_connection_from_fd(int fd, IIOPConnection *parent)
 
   iiop_connection_init(fd_cnx, GIOP_CONNECTION_CLIENT, parent->icnxtype);
 
-  fd_cnx->fd = fd;
+  GIOP_CONNECTION(fd_cnx)->fd = fd;
 
   switch(parent->icnxtype) {
   case IIOP_IPV4:
     n = sizeof(struct sockaddr_in);
-    getpeername(fd_cnx->fd, (struct sockaddr *)&fd_cnx->u.ipv4.location, &n);
+    getpeername(GIOP_CONNECTION_GET_FD(fd_cnx), (struct sockaddr *)&fd_cnx->u.ipv4.location, &n);
     hent = gethostbyaddr((const char *)&fd_cnx->u.ipv4.location.sin_addr.s_addr, 4, AF_INET);
     fd_cnx->u.ipv4.hostname = g_strdup(hent->h_name);
     break;
@@ -406,13 +429,13 @@ iiop_connection_from_fd(int fd, IIOPConnection *parent)
   case IIOP_USOCK:
     n = sizeof(struct sockaddr_un);
     fd_cnx->u.usock.sun_family = AF_UNIX;
-    getpeername(fd_cnx->fd, &fd_cnx->u.usock, &n);
+    getpeername(GIOP_CONNECTION_GET_FD(fd_cnx), &fd_cnx->u.usock, &n);
     break;
 
 #ifdef HAVE_IPV6
   case IIOP_IPV6:
     n = sizeof(struct sockaddr_in6);
-    getpeername(fd_cnx->fd, &fd_cnx->u.ipv6.location, &n);
+    getpeername(GIOP_CONNECTION_GET_FD(fd_cnx), &fd_cnx->u.ipv6.location, &n);
     hent = gethostbyaddr((const char *)&fd_cnx->u.ipv6.location.sin6_addr, 
 			 sizeof(fd_cnx->u.ipv6.location.sin6_addr), AF_INET6);
     fd_cnx->u.ipv6.hostname = g_strdup(hent->h_name);
@@ -423,9 +446,13 @@ iiop_connection_from_fd(int fd, IIOPConnection *parent)
     g_error("Unsupported connection type %d", parent->icnxtype);
   }
 
+  fcntl(GIOP_CONNECTION_GET_FD(fd_cnx), F_SETFD,
+	fcntl(GIOP_CONNECTION_GET_FD(fd_cnx), F_GETFD, 0)
+	| FD_CLOEXEC | O_NONBLOCK);
+
   ORBit_Trace(TraceMod_IIOP, TraceLevel_Debug,
 	      "iiop_connection_from_fd connect [%d]\n",
-	      fd_cnx->fd);
+	      GIOP_CONNECTION_GET_FD(fd_cnx));
 
   giop_connection_add_to_list(GIOP_CONNECTION(fd_cnx));
 
@@ -451,22 +478,23 @@ iiop_connection_server(void)
   iiop_connection_init(server_cnx, GIOP_CONNECTION_SERVER, IIOP_IPV4);
 
   server_cnx->is_serversock = TRUE;
-  server_cnx->fd = socket(PF_INET, SOCK_STREAM, IPPROTO_TCP);
+  GIOP_CONNECTION(server_cnx)->fd = socket(PF_INET, SOCK_STREAM, IPPROTO_TCP);
 
-  if(server_cnx->fd < 0) {
+  if(GIOP_CONNECTION_GET_FD(server_cnx) < 0) {
     ORBit_Trace(TraceMod_IIOP, TraceLevel_Debug, "iiop_connection_new: socket_error: %s\n", strerror(errno));
     goto failed;
   }
 
   server_cnx->u.ipv4.location.sin_family = AF_INET;
   server_cnx->u.ipv4.location.sin_addr.s_addr = INADDR_ANY;
-  bind(server_cnx->fd,
+  bind(GIOP_CONNECTION_GET_FD(server_cnx),
        (struct sockaddr *)&server_cnx->u.ipv4.location,
        sizeof(struct sockaddr_in));
-  fcntl(server_cnx->fd, F_SETFD, fcntl(server_cnx->fd, F_GETFD, 0) | FD_CLOEXEC);
+
+  fcntl(GIOP_CONNECTION_GET_FD(server_cnx), F_SETFD, fcntl(GIOP_CONNECTION_GET_FD(server_cnx), F_GETFD, 0) | FD_CLOEXEC | O_NONBLOCK);
   
   n = sizeof(struct sockaddr_in);
-  getsockname(server_cnx->fd,
+  getsockname(GIOP_CONNECTION_GET_FD(server_cnx),
 	      (struct sockaddr *)&server_cnx->u.ipv4.location, &n);
 
   gethostname(hn_tmp, sizeof(hn_tmp) - 1);
@@ -479,17 +507,17 @@ iiop_connection_server(void)
     else
       server_cnx->u.ipv4.hostname = g_strdup(inet_ntoa(*((struct in_addr *)hent->h_addr)));
   } else
-    server_cnx->u.ipv4.hostname = g_strdup(hn_tmp);    
+    server_cnx->u.ipv4.hostname = g_strdup(hn_tmp);
 
-  listen(server_cnx->fd, 5);
+  listen(GIOP_CONNECTION_GET_FD(server_cnx), 5);
 
   giop_connection_add_to_list(GIOP_CONNECTION(server_cnx));
 
   return server_cnx;
 
 failed:
-  close(server_cnx->fd);
-  server_cnx->fd = -1;
+  close(GIOP_CONNECTION_GET_FD(server_cnx));
+  GIOP_CONNECTION(server_cnx)->fd = -1;
   giop_connection_free(GIOP_CONNECTION(server_cnx));
   server_cnx = NULL;
   /*
@@ -523,22 +551,22 @@ iiop_connection_server_ipv6(void)
   iiop_connection_init(server_cnx, GIOP_CONNECTION_SERVER, IIOP_IPV6);
   
   server_cnx->is_serversock = TRUE;
-  server_cnx->fd = socket(PF_INET6, SOCK_STREAM, IPPROTO_TCP);
+  GIOP_CONNECTION(server_cnx)->fd = socket(PF_INET6, SOCK_STREAM, IPPROTO_TCP);
 
-  if(server_cnx->fd < 0) {
+  if(GIOP_CONNECTION_GET_FD(server_cnx) < 0) {
     ORBit_Trace(TraceMod_IIOP, TraceLevel_Debug, "iiop_connection_new: socket_error: %s\n", strerror(errno));
     goto failed;
   }
 
   server_cnx->u.ipv6.location.sin6_family = AF_INET6;
-  bind(server_cnx->fd,
+  bind(GIOP_CONNECTION_GET_FD(server_cnx),
        (struct sockaddr *)&server_cnx->u.ipv6.location,
        sizeof(struct sockaddr_in6));
   
-  fcntl(server_cnx->fd, F_SETFD, fcntl(server_cnx->fd, F_GETFD, 0) | FD_CLOEXEC);
+  fcntl(GIOP_CONNECTION_GET_FD(server_cnx), F_SETFD, fcntl(GIOP_CONNECTION_GET_FD(server_cnx), F_GETFD, 0) | FD_CLOEXEC | O_NONBLOCK);
 
   n = sizeof(struct sockaddr_in6);
-  getsockname(server_cnx->fd, &server_cnx->u.ipv6.location, &n);
+  getsockname(GIOP_CONNECTION_GET_FD(server_cnx), &server_cnx->u.ipv6.location, &n);
 
   gethostname(hn_tmp, sizeof(hn_tmp) - 1);
 
@@ -552,15 +580,15 @@ iiop_connection_server_ipv6(void)
   } else
     server_cnx->hostname = g_strdup(hn_tmp);
 
-  listen(server_cnx->fd, 5);
+  listen(GIOP_CONNECTION_GET_FD(server_cnx), 5);
 
   giop_connection_add_to_list(GIOP_CONNECTION(server_cnx));
 
   return server_cnx;
 
 failed:
-  close(server_cnx->fd);
-  server_cnx->fd = -1;
+  close(GIOP_CONNECTION_GET_FD(server_cnx));
+  GIOP_CONNECTION(server_cnx)->fd = -1;
   giop_connection_free(GIOP_CONNECTION(server_cnx));
   server_cnx = NULL;
   /*
@@ -595,8 +623,9 @@ iiop_connection_server_unix(const char *sockpath)
   iiop_connection_init(server_cnx_unix, GIOP_CONNECTION_SERVER, IIOP_USOCK);
 
   server_cnx_unix->is_serversock = TRUE;
-  server_cnx_unix->fd = socket(AF_UNIX, SOCK_STREAM, 0);
-  if(server_cnx_unix->fd < 0) {
+  GIOP_CONNECTION(server_cnx_unix)->fd = socket(AF_UNIX, SOCK_STREAM, 0);
+
+  if(GIOP_CONNECTION_GET_FD(server_cnx_unix) < 0) {
     ORBit_Trace(TraceMod_IIOP, TraceLevel_Debug, "iiop_connection_new: socket_error: %s\n", strerror(errno));
     goto failed;
   }
@@ -604,18 +633,19 @@ iiop_connection_server_unix(const char *sockpath)
   strcpy(server_cnx_unix->u.usock.sun_path, sockpath);
 
   server_cnx_unix->u.usock.sun_family = AF_UNIX;
-  if(bind(server_cnx_unix->fd,
+  if(bind(GIOP_CONNECTION_GET_FD(server_cnx_unix),
 	  (struct sockaddr *)&server_cnx_unix->u.usock,
 	  SUN_LEN(&server_cnx_unix->u.usock)) != 0) {
     /* see the comment in iiop_connection_destroy switch as to why we
        close it here. bad hack */
-    close(server_cnx_unix->fd); server_cnx_unix->fd = -1;
+    close(GIOP_CONNECTION_GET_FD(server_cnx_unix));
+    GIOP_CONNECTION(server_cnx_unix)->fd = -1;
     goto failed;
   }
 
-  fcntl(server_cnx_unix->fd, F_SETFD, fcntl(server_cnx_unix->fd, F_GETFD, 0) | FD_CLOEXEC);
+  fcntl(GIOP_CONNECTION_GET_FD(server_cnx_unix), F_SETFD, fcntl(GIOP_CONNECTION_GET_FD(server_cnx_unix), F_GETFD, 0) | FD_CLOEXEC | O_NONBLOCK);
 
-  if(listen(server_cnx_unix->fd, 5) != 0)
+  if(listen(GIOP_CONNECTION_GET_FD(server_cnx_unix), 5) != 0)
     goto failed;
 
   giop_connection_add_to_list(GIOP_CONNECTION(server_cnx_unix));
@@ -630,8 +660,8 @@ iiop_connection_server_unix(const char *sockpath)
   return server_cnx_unix;
 
 failed:
-  close(server_cnx_unix->fd);
-  server_cnx_unix->fd = -1;
+  close(GIOP_CONNECTION_GET_FD(server_cnx_unix));
+  GIOP_CONNECTION(server_cnx_unix)->fd = -1;
   giop_connection_free(GIOP_CONNECTION(server_cnx_unix));
   server_cnx_unix = NULL;
   /*
@@ -752,8 +782,8 @@ iiop_connection_new(const char *host, gushort port)
 
   iiop_connection_init(retval, GIOP_CONNECTION_CLIENT, IIOP_IPV4);
 
-  retval->fd = socket(PF_INET, SOCK_STREAM, IPPROTO_TCP);
-  if(retval->fd < 0) {
+  GIOP_CONNECTION(retval)->fd = socket(PF_INET, SOCK_STREAM, IPPROTO_TCP);
+  if(GIOP_CONNECTION_GET_FD(retval) < 0) {
     ORBit_Trace(TraceMod_IIOP, TraceLevel_Debug, "iiop_connection_new: socket_error: %s\n", strerror(errno));
     goto failed;
   }
@@ -773,14 +803,14 @@ iiop_connection_new(const char *host, gushort port)
       }
       memcpy(&retval->u.ipv4.location.sin_addr, hent->h_addr, (size_t) sizeof(retval->u.ipv4.location.sin_addr));
     }
-  if(connect(retval->fd, (struct sockaddr *)&retval->u.ipv4.location, sizeof(retval->u.ipv4.location)) < 0) {
+  if(connect(GIOP_CONNECTION_GET_FD(retval), (struct sockaddr *)&retval->u.ipv4.location, sizeof(retval->u.ipv4.location)) < 0) {
     ORBit_Trace(TraceMod_IIOP, TraceLevel_Debug, "iiop_connection_new: connect error: %s\n", strerror(errno));
     goto failed;
   }
 
   ORBit_Trace(TraceMod_IIOP, TraceLevel_Debug,
 	      "iiop_connection_new connect [%d] to %s:%d\n",
-	      retval->fd,
+	      GIOP_CONNECTION_GET_FD(retval),
 	      host, (guint)port);
 
   GIOP_CONNECTION(retval)->was_initiated = TRUE;
@@ -790,8 +820,8 @@ iiop_connection_new(const char *host, gushort port)
   return retval;
 
 failed:
-  close(retval->fd);
-  retval->fd = -1;
+  close(GIOP_CONNECTION_GET_FD(retval));
+  GIOP_CONNECTION(retval)->fd = -1;
   giop_connection_free(GIOP_CONNECTION(retval));
   return NULL;
 }
@@ -870,13 +900,13 @@ iiop_connection_unix_new(const char *sockpath)
 
   iiop_connection_init(retval, GIOP_CONNECTION_CLIENT, IIOP_USOCK);
 
-  retval->fd = socket(AF_UNIX, SOCK_STREAM, PF_UNIX);
-  if(retval->fd < 0) {
+  GIOP_CONNECTION(retval)->fd = socket(AF_UNIX, SOCK_STREAM, 0);
+  if(GIOP_CONNECTION_GET_FD(retval) < 0) {
     ORBit_Trace(TraceMod_IIOP, TraceLevel_Debug, "iiop_connection_new: socket_error: %s\n", strerror(errno));
     goto failed;
   }
 
-  if(connect(retval->fd, (struct sockaddr *)&retval->u.usock, SUN_LEN(&retval->u.usock)) < 0) {
+  if(connect(GIOP_CONNECTION_GET_FD(retval), (struct sockaddr *)&retval->u.usock, SUN_LEN(&retval->u.usock)) < 0) {
     ORBit_Trace(TraceMod_IIOP, TraceLevel_Debug, "iiop_connection_new: connect error: %s\n", strerror(errno));
     goto failed;
   }
@@ -887,14 +917,14 @@ iiop_connection_unix_new(const char *sockpath)
 
   ORBit_Trace(TraceMod_IIOP, TraceLevel_Debug,
 	      "iiop_connection_unix_new connect [%d] to %s\n",
-	      retval->fd,
+	      GIOP_CONNECTION_GET_FD(retval),
 	      sockpath);
 
   return retval;
 
 failed:
-  close(retval->fd);
-  retval->fd = -1;
+  close(GIOP_CONNECTION_GET_FD(retval));
+  GIOP_CONNECTION(retval)->fd = -1;
   giop_connection_free(GIOP_CONNECTION(retval));
   return NULL;
 }
@@ -919,7 +949,7 @@ iiop_connection_destroy(IIOPConnection *iiop_connection)
   case IIOP_IPV4:
     ORBit_Trace(TraceMod_IIOP, TraceLevel_Debug,
 		"iiop_connection_destroy connect [%d] of %s:%d\n",
-		iiop_connection->fd,
+		GIOP_CONNECTION_GET_FD(iiop_connection),
 		iiop_connection->u.ipv4.hostname,
 		ntohs(iiop_connection->u.ipv4.location.sin_port));
     g_free(iiop_connection->u.ipv4.hostname);
@@ -938,43 +968,26 @@ iiop_connection_destroy(IIOPConnection *iiop_connection)
        we don't want to unlink the other server's socket!
        if the bind fails, iiop_connection_server_unix closes the fd for us */
     if(GIOP_CONNECTION(iiop_connection)->connection_class == GIOP_CONNECTION_SERVER
-       && IIOP_CONNECTION(iiop_connection)->fd >= 0)
+       && GIOP_CONNECTION(iiop_connection)->fd >= 0)
       unlink(iiop_connection->u.usock.sun_path);
     break;
-  default: ;
+  default:
+    break;
   }
 
-  if(iiop_connection->fd >= 0) {
-
+  if(GIOP_CONNECTION_GET_FD(iiop_connection) >= 0) {
     if(GIOP_CONNECTION(iiop_connection)->is_valid
        && !GIOP_CONNECTION(iiop_connection)->was_initiated)
       {
-	write(iiop_connection->fd, &mh, sizeof(mh));
+	write(GIOP_CONNECTION_GET_FD(iiop_connection), &mh, sizeof(mh));
       }
 
-    shutdown(iiop_connection->fd, 2);
-    close(iiop_connection->fd);
+    shutdown(GIOP_CONNECTION_GET_FD(iiop_connection), 2);
+    close(GIOP_CONNECTION_GET_FD(iiop_connection));
+    GIOP_CONNECTION(iiop_connection)->fd = -1;
   }
 }
 
-/*
- * iiop_get_fd
- *
- *  Inputs: 'giop_connection' - a valid IIOPConnection
- *
- *  Outputs: the file descriptor for the connection.
- *
- *  Description: Implementation of the get_fd method on a GIOPConnection.
- *                Returns the file descriptor for this GIOPConnection.
- */
-static int
-iiop_get_fd(GIOPConnection *giop_connection)
-{
-  g_return_val_if_fail(giop_connection->connection_type == GIOP_CONNECTION_IIOP, -1);
-
-  return IIOP_CONNECTION(giop_connection)->fd;
-}
-
 static int giop_nloops = 0;
 
 void giop_main_quit(void) { giop_nloops--; }
@@ -1035,8 +1048,9 @@ giop_main_next_message_2(gboolean blocking,
 	case IIOP_USOCK: sock.sa_family = AF_UNIX; break;
 	case IIOP_IPV6:
 #ifdef HAVE_IPV6
- sock.sa_family = AF_INET6; break;
+ sock.sa_family = AF_INET6;
 #endif
+	  break;
 	}
 
 	newfd = accept(GIOP_CONNECTION_GET_FD(connection), &sock, &n);
@@ -1074,8 +1088,9 @@ giop_main_handle_connection(GIOPConnection *connection)
     case IIOP_USOCK: sock.sa_family = AF_UNIX; break;
     case IIOP_IPV6:
 #ifdef HAVE_IPV6
-      sock.sa_family = AF_INET6; break;
+      sock.sa_family = AF_INET6;
 #endif
+      break;
     }
 
     newfd = accept(GIOP_CONNECTION_GET_FD(connection), &sock, &n);
@@ -1083,7 +1098,7 @@ giop_main_handle_connection(GIOPConnection *connection)
     newcnx = GIOP_CONNECTION(iiop_connection_from_fd(newfd,
 						     IIOP_CONNECTION(connection)));
     GIOP_CONNECTION(newcnx)->orb_data = connection->orb_data;
-    giop_connection_ref(newcnx);
+
     return;
   } else
     recv_buffer = giop_recv_message_buffer_use(connection);
@@ -1116,8 +1131,21 @@ giop_main_handle_connection_exception(GIOPConnection *connection)
   g_return_if_fail(connection != NULL);
   g_return_if_fail(connection->is_valid);
 
+  giop_connection_ref(connection);
+
   giop_connection_remove_from_list(connection);
+
+  shutdown(GIOP_CONNECTION_GET_FD(connection), 2);
+  close(GIOP_CONNECTION_GET_FD(connection));
+  GIOP_CONNECTION(connection)->fd = -1;
   connection->is_valid = FALSE;
+
+  if(connection->incoming_msg) {
+    giop_recv_buffer_unuse(connection->incoming_msg);
+    connection->incoming_msg = NULL;
+  }
+
+  giop_connection_unref(connection);
 }
 
 /*
