@@ -38,6 +38,12 @@
 
 /* type defs */
 
+#ifdef        __GNUC__
+#define       PACKED __attribute__((packed))
+#else
+#define       PACKED
+#endif
+
 /*
  * Overlaps with struct _GIOPMessageHeader on purpose
  *  - we save time because this stuff never changes
@@ -46,7 +52,7 @@ struct _GIOPMessageHeaderConstants {
   GIOP_char magic[4];
   GIOP_char GIOP_version[2];
   GIOP_octet flags;
-};
+} PACKED;
 
 /* functions */
 static gint giop_recv_decode_message(GIOPRecvBuffer *buf);
@@ -159,43 +165,35 @@ void
 giop_send_buffer_write(GIOPSendBuffer *send_buffer)
 {
   gulong nvecs;
-  gulong res, i, sum;
+  glong res, i, sum, t;
+  struct iovec *curvec;
+  int fd;
+  GIOPConnection *cnx;
 
-#if 0
-  switch(GIOP_MESSAGE_BUFFER(send_buffer)->message_header.message_type) {
-  case GIOP_REQUEST:
-    g_message("[%d] sending request size %d on FD %d",
-	      getpid(),
-	      GIOP_MESSAGE_BUFFER(send_buffer)->message_header.message_size,
-	      GIOP_CONNECTION_GET_FD(GIOP_MESSAGE_BUFFER(send_buffer)->connection));
-    break;
-  case GIOP_REPLY:
-    g_message("[%d] sending reply size %d on FD %d",
-	      getpid(),
-	      GIOP_MESSAGE_BUFFER(send_buffer)->message_header.message_size,
-	      GIOP_CONNECTION_GET_FD(GIOP_MESSAGE_BUFFER(send_buffer)->connection));
-    break;
-  default:
-  }
-#endif
+  cnx = GIOP_MESSAGE_BUFFER(send_buffer)->connection;
+  g_return_if_fail(cnx->is_valid);
 
+  fd = GIOP_CONNECTION_GET_FD(cnx);
   nvecs = GIOP_MESSAGE_BUFFER(send_buffer)->iovecs->len;
+  curvec = (struct iovec *)GIOP_MESSAGE_BUFFER(send_buffer)->iovecs->data;
 
-  res = 
-    writev(GIOP_CONNECTION_GET_FD(GIOP_MESSAGE_BUFFER(send_buffer)->connection),
-	   (struct iovec *)GIOP_MESSAGE_BUFFER(send_buffer)->iovecs->data,
-	   nvecs);
+  res = writev(fd, curvec, nvecs);
 
-  for(i = sum = 0; i < nvecs; i++) {
-    sum += g_array_index((GIOP_MESSAGE_BUFFER(send_buffer)->iovecs), struct iovec, i).iov_len;
-  }
+  for(sum = res; sum < (GIOP_MESSAGE_BUFFER(send_buffer)->message_header.message_size + sizeof(GIOPMessageHeader)); ) {
+    if(res < 0) {
+      giop_main_handle_connection_exception(cnx);
+      return;
+    }
+      
+    for(i = t = 0; t < sum; i++)
+      t += curvec[i].iov_len;
+    curvec = &curvec[i-1];
+    ((char *)curvec->iov_base) += t - sum;
+    curvec->iov_len -= t - sum;
 
-#ifdef DEBUG
-  g_assert(sum == (GIOP_MESSAGE_BUFFER(send_buffer)->message_header.message_size + sizeof(GIOPMessageHeader)));
-#endif
+    nvecs -= i - 1;
 
-  if(sum != res) { /* Connection error */
-    giop_main_handle_connection_exception(GIOP_MESSAGE_BUFFER(send_buffer)->connection);
+    res = writev(fd, curvec, nvecs);
   }
 }
 
@@ -500,20 +498,21 @@ giop_send_buffer_append_mem_indirect(GIOPSendBuffer *send_buffer,
 
     if(thisblock_size >= sizeof(CORBA_unsigned_long)
        || (mem_region_length - offset) < sizeof(CORBA_unsigned_long)) {
-      if(!blockstart)
-	blockstart = send_buffer->indirect + send_buffer->indirect_used;
+      if (!blockstart)
+	(guchar*)blockstart =
+	  ((guchar*) send_buffer->indirect) + send_buffer->indirect_used;
 
-      memcpy(send_buffer->indirect + send_buffer->indirect_used,
-	     mem_region + offset, thisblock_size);
+      memcpy((guchar*)send_buffer->indirect + send_buffer->indirect_used,
+	     (guchar*)mem_region + offset, thisblock_size);
       giop_message_buffer_append_mem(GIOP_MESSAGE_BUFFER(send_buffer),
-				     send_buffer->indirect +
+				     (guchar*)send_buffer->indirect +
 				     send_buffer->indirect_used,
 				     thisblock_size);
       offset += thisblock_size;
       send_buffer->indirect_used += thisblock_size;
     }
 
-    if(send_buffer->indirect_used > GIOP_INDIRECT_CHUNK_SIZE
+    if(send_buffer->indirect_used >= GIOP_INDIRECT_CHUNK_SIZE
        || (thisblock_size >= sizeof(CORBA_unsigned_long)
 	   && (mem_region_length - offset) > 0)) {
 	  send_buffer->indirect_used = 0;
@@ -623,7 +622,11 @@ giop_recv_buffer_unuse(GIOPRecvBuffer *buffer)
   g_return_if_fail(buffer != NULL);
 
   g_free(buffer->message_body);
+  if(GIOP_MESSAGE_BUFFER(buffer)->connection->incoming_msg == buffer)
+    GIOP_MESSAGE_BUFFER(buffer)->connection->incoming_msg = NULL;
+
   giop_connection_unref(GIOP_MESSAGE_BUFFER(buffer)->connection);
+
   GET_LOCK(recvbufferlist);
   recvbufferlist = g_slist_prepend(recvbufferlist, buffer);
   RELEASE_LOCK(recvbufferlist);
@@ -647,13 +650,16 @@ giop_recv_buffer_use(GIOPConnection *connection)
       head = recvbufferlist;
       recvbufferlist = g_slist_remove_link(recvbufferlist, recvbufferlist);
       g_slist_free_1 (head);
-      
+
       GIOP_MESSAGE_BUFFER(retval)->message_header.message_size = 0;
       retval->message_body = NULL;
     }
   else
     retval = giop_recv_buffer_new();
 
+  retval->state = GIOP_MSG_READING_HEADER;
+  retval->left_to_read = sizeof(GIOPMessageHeader);
+
   RELEASE_LOCK(recvbufferlist);
   
   giop_connection_ref(connection);
@@ -666,111 +672,111 @@ GIOPRecvBuffer *
 giop_recv_message_buffer_use(GIOPConnection *connection)
 {
   GIOPRecvBuffer *retval;
+  char *bptr;
   int sysret;
-  int bytes_read;
-  int n_zero_reads;
   int message_size;
   
   if (!connection)
-	  return NULL;
+    return NULL;
 
   g_return_val_if_fail(connection->is_valid, NULL);
 
-  retval = giop_recv_buffer_use(connection);
-
-  if(!retval) return retval;
-
-  bytes_read = n_zero_reads = 0;
-  do
-    {
-      sysret = read(GIOP_CONNECTION_GET_FD(connection),
-		    ((char *)&(GIOP_MESSAGE_BUFFER(retval)->message_header))+bytes_read,
-		    sizeof(GIOP_MESSAGE_BUFFER(retval)->message_header)-bytes_read);
-
-      if(sysret <= 0) {
-	giop_main_handle_connection_exception(connection);
-	giop_recv_buffer_unuse(retval);
-	return NULL;
-      }
-
-      bytes_read += sysret;
-    } while( bytes_read != 
-	     sizeof(GIOP_MESSAGE_BUFFER(retval)->message_header) );
-
-  /* Check the magic stuff */
-  if(strncmp(GIOP_MESSAGE_BUFFER(retval)->message_header.magic, "GIOP", 4)
-     || GIOP_MESSAGE_BUFFER(retval)->message_header.GIOP_version[0] != 1) {
-    giop_main_handle_connection_exception(connection);
-    giop_recv_buffer_unuse(retval);
-    return NULL;
+  if(connection->incoming_msg)
+    retval = connection->incoming_msg;
+  else {
+    retval = giop_recv_buffer_use(connection);
+    connection->incoming_msg = retval;
   }
 
-  if(giop_msg_conversion_needed(GIOP_MESSAGE_BUFFER(retval)))
-    {
-      CORBA_unsigned_long t = GIOP_MESSAGE_BUFFER(retval)->message_header.message_size;
-      retval->decoder = (gpointer)iiop_byteswap;
-      iiop_byteswap((gpointer)&GIOP_MESSAGE_BUFFER(retval)->message_header.message_size,
-	       (gpointer)&t, sizeof(t));
-    }
-  else
-    {
-      retval->decoder = (gpointer)memcpy;
+  if(!retval) return NULL;
+
+  do {
+    switch(retval->state) {
+    case GIOP_MSG_READING_HEADER:
+      bptr = ((char *)&(GIOP_MESSAGE_BUFFER(retval)->message_header));
+      bptr += sizeof(GIOP_MESSAGE_BUFFER(retval)->message_header)
+	- retval->left_to_read;
+      break;
+    case GIOP_MSG_READING_BODY:
+      bptr = retval->message_body;
+      bptr += GIOP_MESSAGE_BUFFER(retval)->message_header.message_size;
+      bptr -= retval->left_to_read;
+      break;
+    default:
+      bptr = NULL;
     }
 
-  if(GIOP_MESSAGE_BUFFER(retval)->message_header.message_size == 0
-       && GIOP_MESSAGE_BUFFER(retval)->message_header.message_type != GIOP_CLOSECONNECTION) {
-    giop_recv_buffer_unuse(retval);
-    g_warning("Unexpected 0-length IIOP message");
-    giop_main_handle_connection_exception(connection);
-    return NULL;
-  }
+    sysret = read(GIOP_CONNECTION_GET_FD(connection), bptr,
+		  retval->left_to_read);
 
-  message_size = GIOP_MESSAGE_BUFFER(retval)->message_header.message_size;
+    if((sysret == 0)
+       || ((sysret < 0) && (errno != EAGAIN)))
+      goto errout;
 
-  /*
-   * We set a limit on the message size on an incoming connection
-   * to avoid a denial of service attack.
-   */
-  
-  if (message_size > 8192 || message_size < 0){
-    g_warning("initial message size is bigger than 8k (%d)", message_size);
-    giop_main_handle_connection_exception (connection);
-    return NULL;
-  }
-  
-  /* Think up a more efficient way of doing this */
-  retval->message_body = g_malloc(message_size);
+    if(sysret > 0)
+      retval->left_to_read -= sysret;
 
-  bytes_read = 0;
-  n_zero_reads = 0;
-  do
-    {
-      sysret = read(GIOP_CONNECTION_GET_FD(connection), 
-		    retval->message_body + bytes_read,
-		    message_size - bytes_read); 
+    if(retval->left_to_read == 0) {
+      /* we change states here */
 
-      if(sysret < 0 || (sysret == 0 && (n_zero_reads++ > 5))) {
-	giop_recv_buffer_unuse(retval);
-	g_warning("read() body [%d] failed = %d: %s",
-		  message_size,
-		  sysret,
-		  g_strerror(errno));
-	giop_main_handle_connection_exception(connection);
-	return NULL;
+      switch(retval->state) {
+      case GIOP_MSG_READING_HEADER:
+	/* Check the magic stuff */
+	if(strncmp(GIOP_MESSAGE_BUFFER(retval)->message_header.magic, "GIOP", 4)
+	   || GIOP_MESSAGE_BUFFER(retval)->message_header.GIOP_version[0] != 1)
+	  goto errout;
+	if(GIOP_MESSAGE_BUFFER(retval)->message_header.message_size == 0
+	   && GIOP_MESSAGE_BUFFER(retval)->message_header.message_type != GIOP_CLOSECONNECTION) {
+	  g_warning("Unexpected 0-length IIOP message");
+	  goto errout;
+	}
+
+	if(giop_msg_conversion_needed(GIOP_MESSAGE_BUFFER(retval))) {
+	  CORBA_unsigned_long t = GIOP_MESSAGE_BUFFER(retval)->message_header.message_size;
+	  retval->decoder = (gpointer)iiop_byteswap;
+
+	  iiop_byteswap((gpointer)&GIOP_MESSAGE_BUFFER(retval)->message_header.message_size,
+			(gpointer)&t, sizeof(t));
+	} else {
+	  retval->decoder = (gpointer)memcpy;
+	}
+
+	message_size = GIOP_MESSAGE_BUFFER(retval)->message_header.message_size;
+	if(!connection->is_auth
+	   && message_size > 131072) {
+	  g_warning("message size is bigger than 128k (%d)", message_size);
+	  goto errout;
+	}
+
+	retval->message_body = g_malloc(message_size);
+	retval->state = GIOP_MSG_READING_BODY;
+	retval->left_to_read = message_size;
+	break;
+      case GIOP_MSG_READING_BODY:
+	retval->cur = retval->message_body;
+	if(giop_recv_decode_message(retval))
+	  goto errout;
+	connection->incoming_msg = NULL;
+	retval->state = GIOP_MSG_READY;
+	break;
+      default:
+        break;
       }
-      bytes_read += sysret;
-    } while( bytes_read != 
-	     message_size);
-
-  retval->cur = retval->message_body;
-
-  if(giop_recv_decode_message(retval))
-    {
-      giop_recv_buffer_unuse(retval);
+    } else if(retval->left_to_read < 0) {
+      g_warning("Whoa, we overstepped the number of bytes we were supposed to read by %d", -retval->left_to_read);
+      goto errout;
+    } else /* retval->left_to_read > 0 */ {
+      /* couldn't read the whole piece, save it */
       retval = NULL;
     }
-
+  } while(retval && retval->state != GIOP_MSG_READY);
+  
   return retval;
+
+ errout:
+  giop_recv_buffer_unuse(retval);
+  giop_main_handle_connection_exception(connection);
+  return NULL;
 }
 
 void
@@ -975,7 +981,7 @@ giop_recv_decode_message(GIOPRecvBuffer *buf)
     }
 }
 
-# define CHECK_POS(buf) g_return_val_if_fail(GIOP_RECV_BUFFER(buf)->cur <= (GIOP_RECV_BUFFER(buf)->message_body + GIOP_MESSAGE_BUFFER(buf)->message_header.message_size), -1)
+# define CHECK_POS(buf) g_return_val_if_fail((guchar*)GIOP_RECV_BUFFER(buf)->cur <= (((guchar *)GIOP_RECV_BUFFER(buf)->message_body) + GIOP_MESSAGE_BUFFER(buf)->message_header.message_size), -1)
 
 /* There be dragons in here. */
 static gint
@@ -1002,9 +1008,12 @@ giop_recv_reply_decode_message(GIOPRecvBuffer *buf)
   if(giop_msg_conversion_needed(GIOP_MESSAGE_BUFFER(buf))) 
     {
 
-#define GET_ULONG(x) ({ iiop_byteswap((guchar *)&(x), \
-			      (guchar *)((buf->cur)),\
-			       sizeof(CORBA_unsigned_long)); buf->cur +=sizeof(CORBA_unsigned_long); })
+#define GET_ULONG(x) G_STMT_START{					\
+		iiop_byteswap ((guchar *)&(x), 				\
+			       (guchar *)(buf->cur),			\
+			        sizeof(CORBA_unsigned_long)); 		\
+		(guchar*) buf->cur += sizeof(CORBA_unsigned_long);	\
+	}G_STMT_END
 
       GET_ULONG(buf->message.u.reply.service_context._length);
       buf->message.u.reply.service_context._buffer =
@@ -1018,7 +1027,7 @@ giop_recv_reply_decode_message(GIOPRecvBuffer *buf)
 	  GET_ULONG(buf->message.u.reply.service_context._buffer[i].context_data._length);
 	  buf->message.u.reply.service_context._buffer[i].context_data._buffer =
 	    buf->cur;
-	  buf->cur += buf->message.u.reply.service_context._buffer[i].context_data._length;
+	  (guchar *) buf->cur += buf->message.u.reply.service_context._buffer[i].context_data._length;
 	}
       CHECK_POS(buf);
       GET_ULONG(buf->message.u.reply.request_id);
@@ -1026,9 +1035,12 @@ giop_recv_reply_decode_message(GIOPRecvBuffer *buf)
     }
   else
     {
-#define GET_ULONG_NC(x) ({ (x) = (*((CORBA_unsigned_long *)(buf->cur))); buf->cur+=sizeof(CORBA_unsigned_long); x; })
+#define GET_ULONG_NC(x) G_STMT_START{ \
+			*(x) = (*((CORBA_unsigned_long *)(buf->cur))); \
+			(guchar *)buf->cur += sizeof(CORBA_unsigned_long); \
+		}G_STMT_END
 
-      GET_ULONG_NC(buf->message.u.reply.service_context._length);
+      GET_ULONG_NC(&buf->message.u.reply.service_context._length);
       buf->message.u.reply.service_context._buffer =
 	g_new(IOP_ServiceContext, buf->message.u.reply.service_context._length);
 
@@ -1036,15 +1048,15 @@ giop_recv_reply_decode_message(GIOPRecvBuffer *buf)
 	{
 	  CHECK_POS(buf);
 	  buf->cur = ALIGN_ADDRESS(buf->cur, 4);
-	  GET_ULONG_NC(buf->message.u.reply.service_context._buffer[i].context_id);
-	  GET_ULONG_NC(buf->message.u.reply.service_context._buffer[i].context_data._length);
+	  GET_ULONG_NC(&buf->message.u.reply.service_context._buffer[i].context_id);
+	  GET_ULONG_NC(&buf->message.u.reply.service_context._buffer[i].context_data._length);
 	  buf->message.u.reply.service_context._buffer[i].context_data._buffer =
 	    buf->cur;
-	  buf->cur += buf->message.u.reply.service_context._buffer[i].context_data._length;
+	  (guchar *) buf->cur += buf->message.u.reply.service_context._buffer[i].context_data._length;
 	}
       CHECK_POS(buf);
-      GET_ULONG_NC(buf->message.u.reply.request_id);
-      GET_ULONG_NC(buf->message.u.reply.reply_status);
+      GET_ULONG_NC(&buf->message.u.reply.request_id);
+      GET_ULONG_NC(&buf->message.u.reply.reply_status);
     }
 
 #undef GET_ULONG  
@@ -1068,19 +1080,22 @@ giop_recv_locate_reply_decode_message(GIOPRecvBuffer *buf)
   if(giop_msg_conversion_needed(GIOP_MESSAGE_BUFFER(buf))) 
     {
 
-#define GET_ULONG(x) ({ iiop_byteswap((guchar *)&(x), \
-			      (guchar *)((buf->cur)),\
-			       sizeof(CORBA_unsigned_long)); buf->cur +=sizeof(CORBA_unsigned_long); })
+#define GET_ULONG(x) G_STMT_START{ \
+			iiop_byteswap((guchar *)&(x),			\
+		          (guchar *)((buf->cur)),			\
+		          sizeof(CORBA_unsigned_long));			\
+		        (guchar *)buf->cur += sizeof(CORBA_unsigned_long); \
+		}G_STMT_END
 
       GET_ULONG(buf->message.u.locate_reply.request_id);
       GET_ULONG(buf->message.u.locate_reply.locate_status);
     }
   else
     {
-#define GET_ULONG_NC(x) ({ (x) = (*((CORBA_unsigned_long *)(buf->cur))); buf->cur+=sizeof(CORBA_unsigned_long); x; })
+#define GET_ULONG_NC(x) G_STMT_START{ *(x) = (*((CORBA_unsigned_long *)(buf->cur))); (guchar*)buf->cur+=sizeof(CORBA_unsigned_long); }G_STMT_END
 
-      GET_ULONG_NC(buf->message.u.locate_reply.request_id);
-      GET_ULONG_NC(buf->message.u.locate_reply.locate_status);
+      GET_ULONG_NC(&buf->message.u.locate_reply.request_id);
+      GET_ULONG_NC(&buf->message.u.locate_reply.locate_status);
     }
 
 #undef GET_ULONG  
@@ -1099,9 +1114,10 @@ giop_recv_request_decode_message(GIOPRecvBuffer *buf)
   if(giop_msg_conversion_needed(GIOP_MESSAGE_BUFFER(buf))) 
     {
 
-#define GET_ULONG(x) ({ iiop_byteswap((guchar *)&(x), \
+#define GET_ULONG(x) G_STMT_START{ iiop_byteswap((guchar *)&(x), \
 			      (guchar *)((buf->cur)),\
-			       sizeof(CORBA_unsigned_long)); buf->cur +=sizeof(CORBA_unsigned_long); })
+			       sizeof(CORBA_unsigned_long)); \
+			      (guchar*) buf->cur +=sizeof(CORBA_unsigned_long); }G_STMT_END
       CHECK_POS(buf);
       GET_ULONG(buf->message.u.request.service_context._length);
       buf->message.u.request.service_context._buffer =
@@ -1114,36 +1130,36 @@ giop_recv_request_decode_message(GIOPRecvBuffer *buf)
 	  GET_ULONG(buf->message.u.request.service_context._buffer[i].context_data._length);
 	  buf->message.u.request.service_context._buffer[i].context_data._buffer =
 	    buf->cur;
-	  buf->cur += buf->message.u.request.service_context._buffer[i].context_data._length;
+	  (guchar*)buf->cur += buf->message.u.request.service_context._buffer[i].context_data._length;
 	}
       CHECK_POS(buf);
       buf->cur = ALIGN_ADDRESS(buf->cur, sizeof(GIOP_unsigned_long));
       GET_ULONG(buf->message.u.request.request_id);
       buf->message.u.request.response_expected = *((GIOP_boolean *)buf->cur);
-      buf->cur += sizeof(GIOP_boolean);
+      (guchar*)buf->cur += sizeof(GIOP_boolean);
       CHECK_POS(buf);
       buf->cur = ALIGN_ADDRESS(buf->cur, sizeof(GIOP_unsigned_long));
       GET_ULONG(buf->message.u.request.object_key._length);
       buf->message.u.request.object_key._buffer = buf->cur;
-      buf->cur += buf->message.u.request.object_key._length;
+      (guchar*)buf->cur += buf->message.u.request.object_key._length;
       CHECK_POS(buf);
       buf->cur = ALIGN_ADDRESS(buf->cur, sizeof(GIOP_unsigned_long));
       GET_ULONG(len);
       buf->message.u.request.operation = buf->cur;
-      buf->cur += len;
+      (guchar*)buf->cur += len;
       CHECK_POS(buf);
       buf->cur = ALIGN_ADDRESS(buf->cur, sizeof(GIOP_unsigned_long));
       GET_ULONG(buf->message.u.request.requesting_principal._length);
       buf->message.u.request.requesting_principal._buffer = buf->cur;      
-      buf->cur += buf->message.u.request.requesting_principal._length;
+      (guchar*)buf->cur += buf->message.u.request.requesting_principal._length;
       CHECK_POS(buf);
     }
   else
     {
-#define GET_ULONG_NC(x) ({ (x) = (*((CORBA_unsigned_long *)(buf->cur))); buf->cur+=sizeof(CORBA_unsigned_long); x; })
+#define GET_ULONG_NC(x) G_STMT_START{ *(x) = (*((CORBA_unsigned_long *)(buf->cur))); (guchar*)buf->cur+=sizeof(CORBA_unsigned_long); }G_STMT_END
 
       CHECK_POS(buf);
-      GET_ULONG_NC(buf->message.u.request.service_context._length);
+      GET_ULONG_NC(&buf->message.u.request.service_context._length);
       buf->message.u.request.service_context._buffer =
 	g_new(IOP_ServiceContext, buf->message.u.request.service_context._length);
 
@@ -1151,31 +1167,31 @@ giop_recv_request_decode_message(GIOPRecvBuffer *buf)
 	{
 	  CHECK_POS(buf);
 	  buf->cur = ALIGN_ADDRESS(buf->cur, 4);
-	  GET_ULONG_NC(buf->message.u.request.service_context._buffer[i].context_id);
-	  GET_ULONG_NC(buf->message.u.request.service_context._buffer[i].context_data._length);
+	  GET_ULONG_NC(&buf->message.u.request.service_context._buffer[i].context_id);
+	  GET_ULONG_NC(&buf->message.u.request.service_context._buffer[i].context_data._length);
 	  buf->message.u.request.service_context._buffer[i].context_data._buffer =
 	    buf->cur;
-	  buf->cur += buf->message.u.request.service_context._buffer[i].context_data._length;
+	  (guchar*)buf->cur += buf->message.u.request.service_context._buffer[i].context_data._length;
 	}
       buf->cur = ALIGN_ADDRESS(buf->cur, sizeof(GIOP_unsigned_long));
-      GET_ULONG_NC(buf->message.u.request.request_id);
+      GET_ULONG_NC(&buf->message.u.request.request_id);
       buf->message.u.request.response_expected = *((GIOP_boolean *)buf->cur);
-      buf->cur += sizeof(GIOP_boolean);
+      (guchar*)buf->cur += sizeof(GIOP_boolean);
       CHECK_POS(buf);
       buf->cur = ALIGN_ADDRESS(buf->cur, sizeof(GIOP_unsigned_long));
-      GET_ULONG_NC(buf->message.u.request.object_key._length);
+      GET_ULONG_NC(&buf->message.u.request.object_key._length);
       buf->message.u.request.object_key._buffer = buf->cur;
-      buf->cur += buf->message.u.request.object_key._length;
+      (guchar*)buf->cur += buf->message.u.request.object_key._length;
       CHECK_POS(buf);
       buf->cur = ALIGN_ADDRESS(buf->cur, sizeof(GIOP_unsigned_long));
-      GET_ULONG_NC(len);
+      GET_ULONG_NC(&len);
       buf->message.u.request.operation = buf->cur;
-      buf->cur += len;
+      (guchar*)buf->cur += len;
       CHECK_POS(buf);
       buf->cur = ALIGN_ADDRESS(buf->cur, sizeof(GIOP_unsigned_long));
-      GET_ULONG_NC(buf->message.u.request.requesting_principal._length);
+      GET_ULONG_NC(&buf->message.u.request.requesting_principal._length);
       buf->message.u.request.requesting_principal._buffer = buf->cur;      
-      buf->cur += buf->message.u.request.requesting_principal._length;
+      (guchar*)buf->cur += buf->message.u.request.requesting_principal._length;
       CHECK_POS(buf);
     }
 
@@ -1199,9 +1215,10 @@ giop_recv_locate_request_decode_message(GIOPRecvBuffer *buf)
   if(giop_msg_conversion_needed(GIOP_MESSAGE_BUFFER(buf))) 
     {
 
-#define GET_ULONG(x) ({ iiop_byteswap((guchar *)&(x), \
+#define GET_ULONG(x) G_STMT_START{ iiop_byteswap((guchar *)&(x), \
 			      (guchar *)((buf->cur)),\
-			       sizeof(CORBA_unsigned_long)); buf->cur +=sizeof(CORBA_unsigned_long); })
+			       sizeof(CORBA_unsigned_long));\
+			       (guchar*)buf->cur +=sizeof(CORBA_unsigned_long); }G_STMT_END
       CHECK_POS(buf);
       buf->cur = ALIGN_ADDRESS(buf->cur, sizeof(GIOP_unsigned_long));
       GET_ULONG(buf->message.u.locate_request.request_id);
@@ -1209,21 +1226,21 @@ giop_recv_locate_request_decode_message(GIOPRecvBuffer *buf)
       buf->cur = ALIGN_ADDRESS(buf->cur, sizeof(GIOP_unsigned_long));
       GET_ULONG(buf->message.u.locate_request.object_key._length);
       buf->message.u.locate_request.object_key._buffer = buf->cur;
-      buf->cur += buf->message.u.locate_request.object_key._length;
+      (guchar*)buf->cur += buf->message.u.locate_request.object_key._length;
       CHECK_POS(buf);
     }
   else
     {
-#define GET_ULONG_NC(x) ({ (x) = (*((CORBA_unsigned_long *)(buf->cur))); buf->cur+=sizeof(CORBA_unsigned_long); x; })
+#define GET_ULONG_NC(x) G_STMT_START{ *(x) = (*((CORBA_unsigned_long *)(buf->cur))); (guchar*)buf->cur+=sizeof(CORBA_unsigned_long); }G_STMT_END
 
       CHECK_POS(buf);
       buf->cur = ALIGN_ADDRESS(buf->cur, sizeof(GIOP_unsigned_long));
-      GET_ULONG_NC(buf->message.u.locate_request.request_id);
+      GET_ULONG_NC(&buf->message.u.locate_request.request_id);
       CHECK_POS(buf);
       buf->cur = ALIGN_ADDRESS(buf->cur, sizeof(GIOP_unsigned_long));
-      GET_ULONG_NC(buf->message.u.locate_request.object_key._length);
+      GET_ULONG_NC(&buf->message.u.locate_request.object_key._length);
       buf->message.u.locate_request.object_key._buffer = buf->cur;
-      buf->cur += buf->message.u.locate_request.object_key._length;
+      (guchar*)buf->cur += buf->message.u.locate_request.object_key._length;
       CHECK_POS(buf);
     }
 
@@ -1247,12 +1264,3 @@ num_on_list(GIOP_unsigned_long num,
 
   return FALSE;
 }
-
-gboolean giop_msg_conversion_needed(GIOPMessageBuffer *msgbuf)
-{
-  gboolean retval;
-
-  retval = conversion_needed(msgbuf->message_header.flags & 1);
-
-  return retval;
-}
