@@ -24,6 +24,26 @@
  *
  */
 
+#define o_return_val_if_fail(expr, val) if(!(expr)) { CORBA_exception_set_system(ev, ex_CORBA_BAD_PARAM, CORBA_COMPLETED_NO); return (val); }
+#define o_return_if_fail(expr) if(!(expr)) { CORBA_exception_set_system(ev, ex_CORBA_BAD_PARAM, CORBA_COMPLETED_NO); return; }
+
+#include "config.h"
+
+/* AIX requires this to be the first thing in the file.  */
+#ifndef __GNUC__
+# if HAVE_ALLOCA_H
+#  include <alloca.h>
+# else
+#  ifdef _AIX
+ #pragma alloca
+#  else
+#   ifndef alloca /* predefined by HP cc +Olibcalls */
+char *alloca ();
+#   endif
+#  endif
+# endif
+#endif
+
 #include <unistd.h>
 #include <errno.h>
 #include <sys/stat.h>
@@ -36,7 +56,9 @@
 #include <time.h>
 #include <sys/types.h>
 #include <dirent.h>
-#include "config.h"
+#include <sys/types.h>
+#include <sys/socket.h>
+#include <sys/un.h>
 #include "../IIOP/iiop-endianP.h"
 #include "orbit.h"
 
@@ -44,6 +66,12 @@
 #include "orbit_object.h"
 #include "orbit_object_type.h"
 
+#ifndef SUN_LEN
+/* This system is not POSIX.1g.  */
+#define SUN_LEN(ptr) ((size_t) (((struct sockaddr_un *) 0)->sun_path)  \
+                      + strlen ((ptr)->sun_path))
+#endif
+
 static void ORBit_ORB_release(CORBA_ORB orb, CORBA_Environment *ev);
 
 static const ORBit_RootObject_Interface CORBA_ORB_epv =
@@ -67,16 +95,24 @@ static int ORBit_ORBid_setup(CORBA_ORB orb, CORBA_ORBid id)
 	return(1);
 }
 
-static void ORBit_rc_load(GString *rcfile, ORBit_orb_options *options)
+static gboolean
+free_key_and_data(gpointer key, gpointer data, gpointer user_data)
+{
+	g_free(key); g_free(data);
+
+	return TRUE;
+}
+
+static void ORBit_rc_load(const char *rcfile, ORBit_orb_options *options)
 {
 	FILE *fp;
 	GHashTable *read_options;
 	ORBit_orb_options *search;
 
-	fp=fopen(rcfile->str, "r");
-	if(fp==NULL) {
+	fp=fopen(rcfile, "r");
+
+	if(fp==NULL)
 		return;
-	}
 
 	read_options=g_hash_table_new(g_str_hash, g_str_equal);
 
@@ -123,6 +159,7 @@ static void ORBit_rc_load(GString *rcfile, ORBit_orb_options *options)
 		}
 	}
 
+	g_hash_table_foreach_remove(read_options, free_key_and_data, NULL);
 	g_hash_table_destroy(read_options);
 }
 
@@ -175,6 +212,7 @@ ORBit_ORB_make_usock_connection(void)
 		int e = errno;
 			
 		switch (e) {
+		case 0:
 		case EEXIST:
 			if (stat(tmpstr->str, &statbuf) != 0)
 				g_error ("Can not stat %s\n", tmpstr->str);
@@ -198,13 +236,28 @@ ORBit_ORB_make_usock_connection(void)
 			 g_get_user_name());
 	dirh = opendir(tmpstr->str);
 	while(!retval && (dent = readdir(dirh))) {
+		int usfd, ret;
+		struct sockaddr_un saddr;
+
+		saddr.sun_family = AF_UNIX;
+
 		if(strncmp(dent->d_name, "orb-", 4))
 			continue;
 
-		g_string_sprintf(tmpstr, "/tmp/orbit-%s/%s",
-				 g_get_user_name(), dent->d_name);
-		retval =
-			GIOP_CONNECTION(iiop_connection_server_unix(tmpstr->str));
+		g_snprintf(saddr.sun_path, sizeof(saddr.sun_path),
+			   "/tmp/orbit-%s/%s",
+			   g_get_user_name(), dent->d_name);
+
+		usfd = socket(AF_UNIX, SOCK_STREAM, 0);
+		g_assert(usfd >= 0);
+
+		ret = connect(usfd, &saddr, SUN_LEN(&saddr));
+		close(usfd);
+
+		if(ret >= 0)
+			continue;
+
+		unlink(saddr.sun_path);
 	}
 	closedir(dirh);
 
@@ -225,8 +278,6 @@ ORBit_ORB_make_usock_connection(void)
  *
  * Adjusts argc and argv appropriately
  */
-static CORBA_ORB orb=NULL;
-
 CORBA_ORB CORBA_ORB_init(int *argc, char **argv, CORBA_ORBid orb_identifier, CORBA_Environment *ev)
 {
 	int no_iiop_server=0;
@@ -243,7 +294,8 @@ CORBA_ORB CORBA_ORB_init(int *argc, char **argv, CORBA_ORBid orb_identifier, COR
 	char *naming_ior=NULL, *naming_addr=NULL;
 	char *root_poa_ior=NULL, *root_poa_addr=NULL;
 	char *orb_id_opt=NULL;
-	GString *tmpstr;
+	char *ctmp;
+	CORBA_ORB orb=NULL;
 
 	/* The variable addresses in these structs need to be assigned at
 	 * run-time if you want to compile with -pedantic
@@ -252,54 +304,73 @@ CORBA_ORB CORBA_ORB_init(int *argc, char **argv, CORBA_ORBid orb_identifier, COR
 	 */
 
 	ORBit_orb_options pre_rc_options[]={
-		{"ORBNoSystemRC", no_arg, &nosysrc},
-		{"ORBNoUserRC", no_arg, &nouserrc},
+		{"ORBNoSystemRC", no_arg, NULL},
+		{"ORBNoUserRC", no_arg, NULL},
 		{NULL, 0, NULL},
 	};
 
 	/* These options are compatible with MICO */
 	ORBit_orb_options options[]={
-		{"ORBNoIIOPServer", no_arg, &no_iiop_server},
-		{"ORBNoIIOPProxy", no_arg, &no_iiop_proxy},
-		{"ORBid", string_arg, &orb_id_opt},
-		{"ORBImplRepoIOR", string_arg, &imr_ior},
-		{"ORBImplRepoAddr", string_arg, &imr_addr},
-		{"ORBIfaceRepoIOR", string_arg, &ir_ior},
-		{"ORBIfaceRepoAddr", string_arg, &ir_addr},
-		{"ORBNamingIOR", string_arg, &naming_ior},
-		{"ORBNamingAddr", string_arg, &naming_addr},
-		{"ORBDebugLevel", int_arg, &debug_level},
+		{"ORBNoIIOPServer", no_arg, NULL},
+		{"ORBNoIIOPProxy", no_arg, NULL},
+		{"ORBid", string_arg, NULL},
+		{"ORBImplRepoIOR", string_arg, NULL},
+		{"ORBImplRepoAddr", string_arg, NULL},
+		{"ORBIfaceRepoIOR", string_arg, NULL},
+		{"ORBIfaceRepoAddr", string_arg, NULL},
+		{"ORBNamingIOR", string_arg, NULL},
+		{"ORBNamingAddr", string_arg, NULL},
+		{"ORBDebugLevel", int_arg, NULL},
 		{"ORBBindAddr", string_arg, NULL}, /* XXX need to make
 						      libIIOP support this */
 		{"ORBIIOPAddr", string_arg, NULL},
 
 	/* These options aren't */
-		{"ORBDebugModules", int_arg, &debug_modules},
-		{"ORBRootPOAIOR", string_arg, &root_poa_ior},
-		{"ORBRootPOAAddr", string_arg, &root_poa_addr},
-		{"ORBIIOPUSock", int_arg, &use_usock},
-		{"ORBIIOPIPv4", int_arg, &use_ipv4},
-		{"ORBIIOPIPv6", int_arg, &use_ipv6},
+		{"ORBDebugModules", int_arg, NULL},
+		{"ORBRootPOAIOR", string_arg, NULL},
+		{"ORBRootPOAAddr", string_arg, NULL},
+		{"ORBIIOPUSock", int_arg, NULL},
+		{"ORBIIOPIPv4", int_arg, NULL},
+		{"ORBIIOPIPv6", int_arg, NULL},
 		{NULL,0,NULL},
 	};
 
-	g_return_val_if_fail(ev != NULL, orb);
-	g_return_val_if_fail(argc != NULL, orb);
-	g_return_val_if_fail(argv != NULL, orb);
+	g_return_val_if_fail(ev != NULL, NULL);
+	o_return_val_if_fail(argc && argv && orb_identifier, NULL);
+
+	pre_rc_options[0].arg = (void *) &nosysrc;
+	pre_rc_options[1].arg = (void *) &nouserrc;
+	options[0].arg = (void *) &no_iiop_server;
+	options[1].arg = (void *) &no_iiop_proxy;
+	options[2].arg = (void *) &orb_id_opt;
+	options[3].arg = (void *) &imr_ior;
+	options[4].arg = (void *) &imr_addr;
+	options[5].arg = (void *) &ir_ior;
+	options[6].arg = (void *) &ir_addr;
+	options[7].arg = (void *) &naming_ior;
+	options[8].arg = (void *) &naming_addr;
+	options[9].arg = (void *) &debug_level;
+	options[12].arg = (void *) &debug_modules;
+	options[13].arg = (void *) &root_poa_ior;
+	options[14].arg = (void *) &root_poa_addr;
+	options[15].arg = (void *) &use_usock;
+	options[16].arg = (void *) &use_ipv4;
+	options[17].arg = (void *) &use_ipv6;
 
 	ORBit_option_parse(argc, argv, pre_rc_options);
 
 	if(!nosysrc) {
-		tmpstr=g_string_new(ORBit_SYSRC);
-		ORBit_rc_load(tmpstr, options);
-		g_string_free(tmpstr, TRUE);
+		ORBit_rc_load(ORBit_SYSRC, options);
 	}
 
 	if(!nouserrc) {
-		tmpstr = g_string_new(NULL);
-		g_string_sprintf(tmpstr, "%s/.orbitrc", g_get_home_dir());
-		ORBit_rc_load(tmpstr, options);
-		g_string_free(tmpstr, TRUE);
+		char *buf;
+
+		ctmp = g_get_home_dir();
+
+		buf = alloca(strlen(ctmp) + sizeof("/.orbitrc"));
+		sprintf(buf, "%s/.orbitrc", ctmp);
+		ORBit_rc_load(buf, options);
 	}
 
 	ORBit_option_parse(argc, argv, options);
@@ -321,7 +392,7 @@ CORBA_ORB CORBA_ORB_init(int *argc, char **argv, CORBA_ORBid orb_identifier, COR
 	}
 
 	ORBit_pseudo_object_init(ORBIT_PSEUDO_OBJECT(orb), ORBIT_PSEUDO_ORB, ev);
-	
+
 	ORBit_RootObject_set_interface(ORBIT_ROOT_OBJECT(orb),
 				       (gpointer)&CORBA_ORB_epv, ev);
 
@@ -347,7 +418,9 @@ CORBA_ORB CORBA_ORB_init(int *argc, char **argv, CORBA_ORBid orb_identifier, COR
 		orb->cnx.ipv4 = GIOP_CONNECTION(iiop_connection_server());
 		giop_connection_ref(orb->cnx.ipv4);
 		GIOP_CONNECTION(orb->cnx.ipv4)->orb_data = orb;
+	
 	}
+
 	if(use_ipv6) {
 		orb->cnx.ipv6 = GIOP_CONNECTION(iiop_connection_server_ipv6());
 		giop_connection_ref(orb->cnx.ipv6);
@@ -355,7 +428,6 @@ CORBA_ORB CORBA_ORB_init(int *argc, char **argv, CORBA_ORBid orb_identifier, COR
 	}
 
 	if(use_usock) {
-
 		orb->cnx.usock = ORBit_ORB_make_usock_connection();
 
 		giop_connection_ref(orb->cnx.usock);
@@ -456,7 +528,6 @@ CORBA_ORB CORBA_ORB_init(int *argc, char **argv, CORBA_ORBid orb_identifier, COR
 		}
 	}
 
-
 	ORBit_custom_run_setup(orb, ev);
 
 	return (CORBA_ORB)CORBA_Object_duplicate((CORBA_Object)orb, ev);
@@ -465,33 +536,16 @@ error:
 	if(orb!=NULL) {
 		ORBit_ORB_release(orb, ev);
 	}
-	if(imr_ior!=NULL) {
-		g_free(imr_ior);
-	}
-	if(imr_addr!=NULL) {
-		g_free(imr_addr);
-	}
-	if(ir_ior!=NULL) {
-		g_free(ir_ior);
-	}
-	if(ir_addr!=NULL) {
-		g_free(ir_addr);
-	}
-	if(naming_ior!=NULL) {
-		g_free(naming_ior);
-	}
-	if(naming_addr!=NULL) {
-		g_free(naming_addr);
-	}
-	if(root_poa_ior!=NULL) {
-		g_free(root_poa_ior);
-	}
-	if(root_poa_addr!=NULL) {
-		g_free(root_poa_addr);
-	}
-	if(orb_id_opt!=NULL) {
-		g_free(orb_id_opt);
-	}
+	g_free(imr_ior);
+	g_free(imr_addr);
+	g_free(ir_ior);
+	g_free(ir_addr);
+	g_free(naming_ior);
+	g_free(naming_addr);
+	g_free(root_poa_ior);
+	g_free(root_poa_addr);
+	g_free(orb_id_opt);
+
 	return(NULL);
 }
 
@@ -506,9 +560,11 @@ static void ORBit_emit_profile(gpointer item, gpointer userdata)
 	ORBit_Object_info *profile=(ORBit_Object_info *)item;
 	profile_user_data *data=(profile_user_data *)userdata;
 	CORBA_Object obj=data->obj;
-	CDR_Codec *codec=data->codec, *encaps;
+	CDR_Codec encaps_codec_d;
+	CDR_Codec *codec=data->codec, *encaps = &encaps_codec_d;
 	gboolean emit_active=data->emit_active;
 	static const CORBA_octet iiopversion[] = {1,0};
+	char codecbuf[2048];
 
 	g_assert(obj!=NULL);
 	g_assert(codec!=NULL);
@@ -519,8 +575,9 @@ static void ORBit_emit_profile(gpointer item, gpointer userdata)
 
 	switch(profile->profile_type) {
 	case IOP_TAG_INTERNET_IOP:
-		encaps = CDR_codec_init();
-		encaps->buffer = g_new(CORBA_octet, 2048);
+		CDR_codec_init_static(encaps);
+		encaps->buffer = codecbuf;
+		encaps->release_buffer = CORBA_FALSE;
 		encaps->buf_len = 2048;
 		encaps->readonly = CORBA_FALSE;
 		encaps->host_endian = encaps->data_endian = FLAG_ENDIANNESS;
@@ -535,12 +592,12 @@ static void ORBit_emit_profile(gpointer item, gpointer userdata)
 			profile->object_key._length);
 		CDR_put_ulong(codec, encaps->wptr);
 		CDR_put_octets(codec, encaps->buffer, encaps->wptr);
-		CDR_codec_free(encaps);
 		break;
 
 	case IOP_TAG_ORBIT_SPECIFIC:
-		encaps = CDR_codec_init();
-		encaps->buffer = g_new(CORBA_octet, 2048);
+		CDR_codec_init_static(encaps);
+		encaps->buffer = codecbuf;
+		encaps->release_buffer = CORBA_FALSE;
 		encaps->buf_len = 2048;
 		encaps->readonly = CORBA_FALSE;
 		encaps->host_endian = encaps->data_endian = FLAG_ENDIANNESS;
@@ -555,7 +612,6 @@ static void ORBit_emit_profile(gpointer item, gpointer userdata)
 			profile->object_key._length);
 		CDR_put_ulong(codec, encaps->wptr);
 		CDR_put_octets(codec, encaps->buffer, encaps->wptr);
-		CDR_codec_free(encaps);
 		break;
 
 	default:
@@ -569,11 +625,16 @@ CORBA_char *CORBA_ORB_object_to_string(CORBA_ORB orb,
 				       CORBA_Environment *ev)
 {
   int i;
-  CDR_Codec *codec;
-  GString *retval;
+  CDR_Codec codec_d;
+  CDR_Codec *codec = &codec_d;
   CORBA_char *rc = NULL;
   CORBA_unsigned_long ntags;
   profile_user_data data;
+  char codecbuf[2048];
+  char *ctmp;
+
+  g_return_val_if_fail(ev, NULL);
+  o_return_val_if_fail(orb && obj, NULL);
 
   if(!obj || !orb) {
 	  CORBA_exception_set_system(ev,
@@ -589,9 +650,10 @@ CORBA_char *CORBA_ORB_object_to_string(CORBA_ORB orb,
 	  return NULL;
   }
 
-  codec = CDR_codec_init();
+  CDR_codec_init_static(codec);
 
-  codec->buffer = g_new(CORBA_octet, 2048);
+  codec->buffer = codecbuf;
+  codec->release_buffer = CORBA_FALSE;
   codec->buf_len = 2048;
   codec->readonly = CORBA_FALSE;
   codec->host_endian = codec->data_endian = FLAG_ENDIANNESS;
@@ -611,16 +673,16 @@ CORBA_char *CORBA_ORB_object_to_string(CORBA_ORB orb,
   data.emit_active=FALSE;
   g_slist_foreach(obj->profile_list, ORBit_emit_profile, &data);
 
-  retval = g_string_new("IOR:");
-  for(i = 0; i < codec->wptr; i++)
-    {
-      g_string_sprintfa(retval, "%02x", codec->buffer[i]);
-    }
+  rc = CORBA_string_alloc(4 + (codec->wptr * 2) + 1);
+  strcpy(rc, "IOR:");
 
-  rc = CORBA_string_dup(retval->str);
-  g_string_free(retval, TRUE);
+#define hexdigit(n) (((n)>9)?(n+'a'-10):(n+'0'))
 
-  CDR_codec_free(codec);
+  for(i = 0, ctmp = rc + strlen("IOR:"); i < codec->wptr; i++) {
+	  *(ctmp++) = hexdigit((((codec->buffer[i]) & 0xF0) >> 4));
+	  *(ctmp++) = hexdigit(((codec->buffer[i]) & 0xF));
+  }
+  *ctmp = '\0';
 
   return rc;
 }
@@ -632,18 +694,22 @@ CORBA_Object CORBA_ORB_string_to_object(CORBA_ORB orb, CORBA_char *str,
 	CORBA_Object retval = NULL;
 	CORBA_char *type_id;
 	ORBit_Object_info *object_info;
-	CDR_Codec *codec, *encaps_codec = NULL;
+	CDR_Codec codec_d, encaps_codec_d;
+	CDR_Codec *codec = &codec_d, *encaps_codec = &encaps_codec_d;
 	CORBA_octet *buffer, endian;
 	int i, j;
 	CORBA_unsigned_long len, seq_len, misclen;
 
+	g_return_val_if_fail(ev, CORBA_OBJECT_NIL);
+	o_return_val_if_fail(orb && str, CORBA_OBJECT_NIL);
+
 	if(strncmp(str, "IOR:", 4)) {
 		CORBA_exception_set_system(ev, ex_CORBA_MARSHAL,
 					   CORBA_COMPLETED_NO);
 		return(CORBA_OBJECT_NIL);
 	}
 
-	codec = CDR_codec_init();
+	CDR_codec_init_static(codec);
 	len = strlen(str);
 
 	if((len % 2) || len <= 4) {
@@ -653,15 +719,10 @@ CORBA_Object CORBA_ORB_string_to_object(CORBA_ORB orb, CORBA_char *str,
 	}
 
 	codec->buf_len = (len-4)/2;
-	buffer = g_new(CORBA_octet, codec->buf_len);
-
-	if(!buffer) {
-		CORBA_exception_set_system(ev, ex_CORBA_NO_MEMORY,
-					   CORBA_COMPLETED_NO);
-		return(CORBA_OBJECT_NIL);
-	}
+	buffer = alloca(codec->buf_len);
 
 	codec->buffer=buffer;
+	codec->release_buffer = CORBA_FALSE;
 	codec->readonly = TRUE;
 
 	for(j = 0, i = 4; i < len; i+=2) {
@@ -673,7 +734,7 @@ CORBA_Object CORBA_ORB_string_to_object(CORBA_ORB orb, CORBA_char *str,
 	codec->data_endian = endian;
 	codec->host_endian = FLAG_ENDIANNESS;
 
-	CDR_get_string(codec, &type_id);
+	CDR_get_string_static(codec, &type_id);
 
 	CDR_get_seq_begin(codec, &seq_len);
 
@@ -686,8 +747,9 @@ CORBA_Object CORBA_ORB_string_to_object(CORBA_ORB orb, CORBA_char *str,
 		switch(tag) {
 		case IOP_TAG_INTERNET_IOP:
 			CDR_get_ulong(codec, &misclen);
-			encaps_codec = CDR_codec_init();
-			encaps_codec->buffer = g_new(CORBA_octet, misclen);
+			CDR_codec_init_static(encaps_codec);
+			encaps_codec->buffer = alloca(misclen);
+			encaps_codec->release_buffer = FALSE;
 			CDR_buffer_gets(codec, encaps_codec->buffer, misclen);
 
 			encaps_codec->buf_len = misclen;
@@ -712,26 +774,25 @@ CORBA_Object CORBA_ORB_string_to_object(CORBA_ORB orb, CORBA_char *str,
 					object_info->object_key._length);
 			ORBit_set_object_key(object_info);
 			profiles=g_slist_append(profiles, object_info);
-			CDR_codec_free(encaps_codec);
-			encaps_codec = NULL;
 			break;
 
 		case IOP_TAG_MULTIPLE_COMPONENTS:
 			/* Just skip any multiple_components data, for now */
 			CDR_get_ulong(codec, &misclen);
-			encaps_codec = CDR_codec_init();
+			CDR_codec_init_static(encaps_codec);
 			encaps_codec->buf_len = misclen;
-			encaps_codec->buffer = g_new(CORBA_octet, misclen);
+			encaps_codec->buffer = alloca(misclen);
+			encaps_codec->release_buffer = FALSE;
 			encaps_codec->readonly = CORBA_TRUE;
 			CDR_buffer_gets(codec, encaps_codec->buffer, misclen);
-			CDR_codec_free(encaps_codec); encaps_codec = NULL;
 			break;
 
 		case IOP_TAG_ORBIT_SPECIFIC:
 			CDR_get_ulong(codec, &misclen);
-			encaps_codec = CDR_codec_init();
+			CDR_codec_init_static(encaps_codec);
 
-			encaps_codec->buffer = g_new(CORBA_octet, misclen);
+			encaps_codec->buffer = alloca(misclen);
+			encaps_codec->release_buffer = FALSE;
 			CDR_buffer_gets(codec, encaps_codec->buffer, misclen);
 
 			encaps_codec->buf_len = misclen;
@@ -757,19 +818,18 @@ CORBA_Object CORBA_ORB_string_to_object(CORBA_ORB orb, CORBA_char *str,
 					object_info->object_key._length);
 			ORBit_set_object_key(object_info);
 			profiles=g_slist_append(profiles, object_info);
-			CDR_codec_free(encaps_codec); encaps_codec = NULL;
 			break;
 		default:
 			g_warning("Unknown tag 0x%x", tag);
 
 			/* Skip it */
 			CDR_get_ulong(codec, &misclen);
-			encaps_codec = CDR_codec_init();
+			CDR_codec_init_static(encaps_codec);
 			encaps_codec->buf_len = misclen;
-			encaps_codec->buffer = g_new(CORBA_octet, misclen);
+			encaps_codec->buffer = alloca(misclen);
+			encaps_codec->release_buffer = FALSE;
 			encaps_codec->readonly = CORBA_TRUE;
 			CDR_buffer_gets(codec, encaps_codec->buffer, misclen);
-			CDR_codec_free(encaps_codec); encaps_codec = NULL;
 			break;
 		}
 	}
@@ -777,10 +837,6 @@ CORBA_Object CORBA_ORB_string_to_object(CORBA_ORB orb, CORBA_char *str,
 	retval = ORBit_create_object_with_info(profiles, type_id, orb, ev);
 
  error_out:
-	g_free(type_id);
-	if(encaps_codec)
-		CDR_codec_free(encaps_codec);
-	CDR_codec_free(codec);
 
 	return retval;
 }
@@ -789,13 +845,7 @@ CORBA_Object CORBA_ORB_string_to_object(CORBA_ORB orb, CORBA_char *str,
 CORBA_Status CORBA_ORB_get_default_context(CORBA_ORB orb, CORBA_Context *ctx, CORBA_Environment *ev)
 {
 	g_return_if_fail(ev != NULL);
-
-	if(!ctx) {
-		CORBA_exception_set_system(ev,
-					   ex_CORBA_BAD_PARAM,
-					   CORBA_COMPLETED_NO);
-		return;
-	}
+	o_return_if_fail(orb && ctx);
 
 	/* I have no clue what this is supposed to be...
 	   v2.2 spec page 5-13 says a CORBA_Context is a list of
@@ -815,6 +865,9 @@ CORBA_boolean CORBA_ORB_get_service_information(CORBA_ORB orb, CORBA_ServiceType
 
 CORBA_Current *CORBA_ORB_get_current(CORBA_ORB orb, CORBA_Environment *ev)
 {
+	g_return_val_if_fail(ev, NULL);
+	o_return_val_if_fail(orb, NULL);
+
 	/* XXX check this over */
 	return (CORBA_Current *)GET_THREAD_DATA();
 }
@@ -822,25 +875,16 @@ CORBA_Current *CORBA_ORB_get_current(CORBA_ORB orb, CORBA_Environment *ev)
 /* Section 4.5 */
 CORBA_ORB_ObjectIdList *CORBA_ORB_list_initial_services(CORBA_ORB orb, CORBA_Environment *ev)
 {
+	static const char *services[] = {"RootPOA"};
 	CORBA_ORB_ObjectIdList *list;
 
-	g_assert(orb!=NULL);
+	g_return_val_if_fail(ev, NULL);
+	o_return_val_if_fail(orb, NULL);
 
-	list= g_new0(CORBA_ORB_ObjectIdList, 1);
-	if(list==NULL) {
-		CORBA_exception_set_system(ev, ex_CORBA_NO_MEMORY, CORBA_COMPLETED_NO);
-		return(NULL);
-	}
-
-	list->_maximum=2;
-	list->_length=2;
-	list->_buffer=g_new(CORBA_ORB_ObjectId, 2);
-	if(list->_buffer==NULL) {
-		CORBA_exception_set_system(ev, ex_CORBA_NO_MEMORY, CORBA_COMPLETED_NO);
-		g_free(list);
-		return(NULL);
-	}
-	CORBA_sequence_set_release((void *)list, CORBA_TRUE);
+	list = (CORBA_ORB_ObjectIdList *)CORBA_sequence_octet__alloc();
+	list->_maximum=list->_length= 1;
+	list->_buffer = (CORBA_ORB_ObjectId *)services;
+	CORBA_sequence_set_release((void *)list, CORBA_FALSE);
 
 	/* defined reserved references are:
 	 *	RootPOA
@@ -851,10 +895,8 @@ CORBA_ORB_ObjectIdList *CORBA_ORB_list_initial_services(CORBA_ORB orb, CORBA_Env
 	 *	SecurityCurrent
 	 *	TransactionCurrent
 	 */
-	list->_buffer[0]="ImplementationRepository";
-	list->_buffer[1]="RootPOA";
 
-	return(list);
+	return list;
 }
 
 /* Section 4.5
@@ -863,8 +905,8 @@ CORBA_ORB_ObjectIdList *CORBA_ORB_list_initial_services(CORBA_ORB orb, CORBA_Env
  */
 CORBA_Object CORBA_ORB_resolve_initial_references(CORBA_ORB orb, CORBA_ORB_ObjectId identifier, CORBA_Environment *ev)
 {
-	g_assert(orb!=NULL);
-	g_assert(identifier!=NULL);
+	g_return_val_if_fail(ev, CORBA_OBJECT_NIL);
+	o_return_val_if_fail(orb, CORBA_OBJECT_NIL);
 
 	if(!strcmp(identifier, "ImplementationRepository"))
 		return CORBA_Object_duplicate(orb->imr, ev);
@@ -881,17 +923,22 @@ CORBA_Object CORBA_ORB_resolve_initial_references(CORBA_ORB orb, CORBA_ORB_Objec
 			poa_mgr->orb = orb;
 
 			/* Create the root poa */
-			orb->root_poa = ORBit_POA_new(orb, "RootPOA",
-						      poa_mgr,
-						      &policies,
-						      ev);
+			orb->root_poa = (CORBA_Object)
+				ORBit_POA_new(orb,
+					      "RootPOA",
+					      poa_mgr,
+					      &policies,
+					      ev);
 		}
 
 		return CORBA_Object_duplicate(orb->root_poa, ev);
 	}
 
 	/* throw user exception: InvalidName */
-	CORBA_exception_set(ev,CORBA_USER_EXCEPTION,ex_CORBA_ORB_InvalidName,NULL);	
+	CORBA_exception_set(ev,CORBA_USER_EXCEPTION,
+			    ex_CORBA_ORB_InvalidName,
+			    NULL);
+
 	goto error;
 error:
 	return(NULL);
@@ -903,9 +950,8 @@ error:
  */
 void CORBA_ORB_set_initial_reference(CORBA_ORB orb, CORBA_ORB_ObjectId identifier, CORBA_Object obj, CORBA_Environment *ev)
 {
-	g_assert(orb!=NULL);
-	g_assert(identifier!=NULL);
-	g_assert(obj!=NULL);
+	g_return_if_fail(ev);
+	o_return_if_fail(orb && identifier && obj);
 
 	if(!strcmp(identifier, "ImplementationRepository")) {
 		if(!CORBA_Object_is_nil(orb->imr, ev)) {
@@ -958,6 +1004,9 @@ CORBA_ORB_shutdown(CORBA_ORB orb,
 		   CORBA_boolean wait_for_completion,
 		   CORBA_Environment *ev)
 {
+	g_return_if_fail(ev);
+	o_return_if_fail(orb);
+
 	/* XXX implement on a per-ORB basis, and also
 	   handle whatever wait_for_completion means */
 
@@ -978,12 +1027,18 @@ CORBA_ORB_shutdown(CORBA_ORB orb,
 CORBA_PolicyType
 CORBA_Policy__get_policy_type(CORBA_Policy obj, CORBA_Environment *ev)
 {
+	g_return_val_if_fail(ev, 0);
+	o_return_val_if_fail(obj, 0);
+
 	return obj->policy_type;
 }
 
 /* Section 4.7 */
 CORBA_Policy CORBA_Policy_copy(CORBA_Policy obj, CORBA_Environment *ev)
 {
+	g_return_val_if_fail(ev, CORBA_OBJECT_NIL);
+	o_return_val_if_fail(obj, CORBA_OBJECT_NIL);
+
 	ORBIT_ROOT_OBJECT_REF(obj);
 
 	return obj;
@@ -995,6 +1050,9 @@ CORBA_Policy CORBA_Policy_copy(CORBA_Policy obj, CORBA_Environment *ev)
  */
 void CORBA_Policy_destroy(CORBA_Policy obj, CORBA_Environment *ev)
 {
+	g_return_if_fail(ev);
+	o_return_if_fail(obj);
+
 	ORBIT_ROOT_OBJECT_UNREF(obj);
 	if(ORBIT_ROOT_OBJECT(obj)->refs <= 0)
 		ORBIT_ROOT_OBJECT_release(obj, ev);
@@ -1003,6 +1061,9 @@ void CORBA_Policy_destroy(CORBA_Policy obj, CORBA_Environment *ev)
 /* Section 4.8.2 */
 CORBA_Policy CORBA_DomainManager_get_domain_policy(CORBA_DomainManager obj, CORBA_PolicyType policy_type, CORBA_Environment *ev)
 {
+	g_return_val_if_fail(ev, CORBA_OBJECT_NIL);
+	o_return_val_if_fail(obj, CORBA_OBJECT_NIL);
+
 	g_assert(!"Not yet implemented");
 	return(NULL);
 }
@@ -1011,6 +1072,9 @@ CORBA_Policy CORBA_DomainManager_get_domain_policy(CORBA_DomainManager obj, CORB
 void CORBA_ConstructionPolicy_make_domain_manager(CORBA_ConstructionPolicy obj, CORBA_InterfaceDef object_type, CORBA_boolean constr_policy, CORBA_Environment *
 ev)
 {
+	g_return_if_fail(ev);
+	o_return_if_fail(obj && object_type);
+
 	g_assert(!"Not yet implemented");
 	return;
 }
@@ -1018,6 +1082,9 @@ ev)
 /* Section 4.2.8 */
 CORBA_DomainManagerList *CORBA_Object_get_domain_managers(CORBA_Object obj, CORBA_Environment *ev)
 {
+	g_return_val_if_fail(ev, NULL);
+	o_return_val_if_fail(obj, NULL);
+
 	g_assert(!"Not yet implemented");
 	return(NULL);
 }
@@ -1025,6 +1092,9 @@ CORBA_DomainManagerList *CORBA_Object_get_domain_managers(CORBA_Object obj, CORB
 /* Section 7.2.2 */
 CORBA_DynAny *CORBA_ORB_create_dyn_any(CORBA_ORB obj, CORBA_any value, CORBA_Environment *ev)
 {
+	g_return_val_if_fail(ev, NULL);
+	o_return_val_if_fail(obj, NULL);
+
 	g_assert(!"Not yet implemented");
 	return(NULL);
 }
