@@ -146,7 +146,7 @@ gpointer
 CORBA_TypeCode__free(gpointer mem, gpointer func_data, CORBA_boolean free_strings)
 {
 	CORBA_Object_release(*(CORBA_Object *)mem, NULL);
-	return ((char *)mem) + sizeof(CORBA_TypeCode);
+	return ((guchar *)mem) + sizeof(CORBA_TypeCode);
 }
 
 CORBA_char *CORBA_string_dup(CORBA_char *string)
@@ -172,7 +172,7 @@ CORBA_string__free(gpointer str, gpointer dat, CORBA_boolean free_strings)
 {
 	if(free_strings)
 		CORBA_free(*((gpointer *)str));
-	return str + sizeof(CORBA_char *);
+	return (gpointer)((guchar *)str + sizeof(CORBA_char *));
 }
 
 gpointer CORBA_Object__free(gpointer str, gpointer dat, CORBA_boolean free_strings)
@@ -181,7 +181,7 @@ gpointer CORBA_Object__free(gpointer str, gpointer dat, CORBA_boolean free_strin
 	CORBA_exception_init(&ev);
 	CORBA_Object_release(*((gpointer *)str), &ev);
 	CORBA_exception_free(&ev);
-	return str + sizeof(CORBA_Object);
+	return (gpointer)((guchar *)str + sizeof(CORBA_Object));
 }
 
 /* 19.14 */
@@ -235,11 +235,16 @@ CORBA_long CORBA_fixed_fraction_part(const void *fp)
 	return retval;
 }
 
-#define do_div(n) ({ \
-int __res; \
-__res = ((CORBA_long) n) % (unsigned) 10; \
-n = ((CORBA_long) n) / (unsigned) 10; \
-__res; })
+static inline
+CORBA_long do_div (CORBA_long *n)
+{
+  int __res;
+
+  __res = (*n) % (unsigned) 10;
+  *n = (*n) / (unsigned) 10;
+
+  return __res;
+}
 
 void CORBA_fixed_set(void *rp, CORBA_long i, CORBA_long f)
 {
@@ -258,7 +263,7 @@ void CORBA_fixed_set(void *rp, CORBA_long i, CORBA_long f)
 
 	for(cur = 0, left_to_eat = i;
 	    left_to_eat != 0 && cur < val->_digits; cur++) {
-		val->_value[cur] = do_div(left_to_eat) * sign;
+		val->_value[cur] = do_div(&left_to_eat) * sign;
 		sign = 1;
 	}
 
@@ -266,7 +271,7 @@ void CORBA_fixed_set(void *rp, CORBA_long i, CORBA_long f)
 
 	for(left_to_eat = f;
 	    left_to_eat != 0 && cur < val->_digits; cur++) {
-		val->_value[cur] = do_div(left_to_eat);
+		val->_value[cur] = do_div(&left_to_eat);
 	}
 }
 
@@ -348,7 +353,7 @@ void ORBit_set_default_principal(CORBA_Principal *principal)
 
 	memcpy(t, &principal->_length, sizeof(principal->_length));
 
-	t += sizeof(principal->_length);
+	(guchar *)t += sizeof(principal->_length);
 	memcpy(t, principal->_buffer, principal->_length);
 }
 
@@ -356,7 +361,7 @@ CORBA_unsigned_long ORBit_class_assignment_counter = 0;
 
 /* XXX not thread-safe */
 CORBA_unsigned_long
-ORBit_register_class(PortableServer_ClassInfo *class_info)
+ORBit_register_class(const PortableServer_ClassInfo *class_info)
 {
 	static GHashTable *class_assignments = NULL;
 	CORBA_unsigned_long retval;
