@@ -31,8 +31,6 @@
 #ifndef _ORBIT_ORBIT_POA_TYPE_H_
 #define _ORBIT_ORBIT_POA_TYPE_H_
 
-#include "orbit_poa.h"
-
 typedef void (*ORBitSkeleton)(PortableServer_ServantBase *_ORBIT_servant,
 			      gpointer _ORBIT_recv_buffer,
 			      CORBA_Environment *ev,
@@ -50,6 +48,7 @@ typedef struct {
 	PortableServer_Servant servant;
 	PortableServer_POA poa;
 	CORBA_ORB orb;
+	CORBA_unsigned_long objnum;
 } ORBit_POAObject;
 
 typedef struct {
@@ -68,6 +67,8 @@ struct PortableServer_POA_type {
 	CORBA_unsigned_long poaID;
 
 	GHashTable *active_object_map;
+	GPtrArray *objnum_to_obj; /* maps objnums to ORBit_POAObject's */
+	CORBA_unsigned_long first_free_id;
 
 	/* Requests received while in a HOLDING state */
 	GSList *held_requests;
