@@ -113,13 +113,17 @@ static const TkInfo tk_info[CORBA_tk_last]={
 
 void ORBit_encode_CORBA_TypeCode(CORBA_TypeCode t, GIOPSendBuffer* buf)
 {
-	CDR_Codec* codec;
+        CDR_Codec codec_d;
+	CDR_Codec* codec = &codec_d;
 	TCEncodeContext ctx;
 	GSList* l;
-	codec=CDR_codec_init();
+	char codecbuf[2048];
+
+	CDR_codec_init_static(codec);
 
 	codec->wptr = 0;
-	codec->buffer = g_malloc(2048); /* xxx bad bad hack */
+	codec->buffer = codecbuf;
+	codec->release_buffer = FALSE;
 	codec->buf_len = 2048;
 	codec->data_endian=FLAG_ENDIANNESS;
 
@@ -132,20 +136,23 @@ void ORBit_encode_CORBA_TypeCode(CORBA_TypeCode t, GIOPSendBuffer* buf)
 	giop_send_buffer_append_mem_indirect(buf,
 					     codec->buffer,
 					     codec->wptr);
-	CDR_codec_free(codec);
 }
 
 void ORBit_decode_CORBA_TypeCode(CORBA_TypeCode* t, GIOPRecvBuffer* buf)
 {
-	CDR_Codec* codec;
+        CDR_Codec codec_d;
+	CDR_Codec* codec = &codec_d;
 	TCDecodeContext ctx;
 	GSList* l;
 
-	codec=CDR_codec_init();
+	CDR_codec_init_static(codec);
 	codec->buffer=buf->cur;
 	codec->release_buffer=CORBA_FALSE;
 	codec->readonly=CORBA_TRUE;
-	codec->buf_len=GIOP_MESSAGE_BUFFER(buf)->message_header.message_size+buf->message_body-buf->cur; /* Hope this is correct */
+	codec->buf_len =		/* hope this is correct */
+		GIOP_MESSAGE_BUFFER(buf)->message_header.message_size +
+		(((guchar *)buf->message_body) - ((guchar *)buf->cur));
+		
 	codec->data_endian=GIOP_MESSAGE_BUFFER(buf)->message_header.flags & 1;
 
 	ctx.current_idx=0;
@@ -154,8 +161,7 @@ void ORBit_decode_CORBA_TypeCode(CORBA_TypeCode* t, GIOPRecvBuffer* buf)
 	for(l=ctx.prior_tcs;l;l=l->next)
 		g_free(l->data);
 	g_slist_free(ctx.prior_tcs);
-	buf->cur += codec->rptr;
-	CDR_codec_free(codec);
+	((guchar *)buf->cur) += codec->rptr;
 }
 	
 
@@ -168,6 +174,9 @@ static void tc_enc(CORBA_TypeCode tc,
 	TCRecursionNode* node;
 	const TkInfo* info;
 	GSList* l;
+	char codecbuf[2048];
+	CDR_Codec encaps_d;
+	CDR_Codec* encaps = &encaps_d;
 
 	g_assert(tc->kind < CORBA_tk_last);
 
@@ -195,15 +204,16 @@ static void tc_enc(CORBA_TypeCode tc,
 
 	info=&tk_info[tc->kind];
 	switch(info->type){
-		CDR_Codec* encaps;
 		guint tmp_index;
 	case TK_EMPTY:
 		break;
 	case TK_COMPLEX:
 		tmp_index=ctx->current_idx;
 		ctx->current_idx+=codec->wptr+4; /* +4 for the length */
-		encaps = CDR_codec_init();
-		encaps->wptr = 0; encaps->buffer = g_malloc(2048);
+		CDR_codec_init_static(encaps);
+		encaps->wptr = 0;
+		encaps->buffer = codecbuf;
+		encaps->release_buffer = FALSE;
 		encaps->buf_len = 2048;
 		encaps->data_endian=FLAG_ENDIANNESS;
 		CDR_put_octet(encaps, FLAG_ENDIANNESS);
@@ -211,7 +221,6 @@ static void tc_enc(CORBA_TypeCode tc,
 		CDR_put_ulong(codec, encaps->wptr);
 		/* Now this is a time hog */ 
 		CDR_put_octets(codec, encaps->buffer, encaps->wptr);
-		CDR_codec_free(encaps);
 		ctx->current_idx=tmp_index;
 		break;
 	case TK_SIMPLE:
@@ -267,6 +276,8 @@ static void tc_dec(CORBA_TypeCode* t, CDR_Codec* c, TCDecodeContext* ctx)
 	CORBA_TypeCode tc;
 	const TkInfo* info;
 	TCRecursionNode* node;
+	CDR_Codec encaps_d;
+	CDR_Codec* encaps = &encaps_d;
 
 	CDR_get_ulong(c, &kind);
 
@@ -307,14 +318,13 @@ static void tc_dec(CORBA_TypeCode* t, CDR_Codec* c, TCDecodeContext* ctx)
 
 	tc->kind=kind;
 	switch(info->type){
-		CDR_Codec* encaps;
 		guint tmp_index;
 		CORBA_octet o;
 	case TK_EMPTY:
 		break;
 	case TK_COMPLEX:
 		tmp_index=ctx->current_idx;
-		encaps=CDR_codec_init();
+		CDR_codec_init_static(encaps);
 		CDR_get_ulong(c, &encaps->buf_len);
 		ctx->current_idx+=c->rptr;
 		encaps->buffer=&c->buffer[c->rptr];
@@ -323,7 +333,6 @@ static void tc_dec(CORBA_TypeCode* t, CDR_Codec* c, TCDecodeContext* ctx)
 		encaps->data_endian=o;
 		(info->decoder)(tc, encaps, ctx);
 		c->rptr += encaps->buf_len;
-		CDR_codec_free(encaps);
 		ctx->current_idx=tmp_index;
 		break;
 	case TK_SIMPLE:
