@@ -29,6 +29,7 @@
 #include <assert.h>
 
 #include "orbit.h"
+#include "orbit_poa.h"
 
 PortableServer_ThreadPolicyValue
 PortableServer_ThreadPolicy__get_value(PortableServer_ThreadPolicy  obj, CORBA_Environment *ev)
@@ -653,7 +654,25 @@ void PortableServer_POA_set_servant(PortableServer_POA obj, PortableServer_Serva
 	obj->default_servant = p_servant;
 }
 
-CORBA_ORB
+static CORBA_unsigned_long
+get_objnum_for_obj(PortableServer_POA poa, ORBit_POAObject *obj)
+{
+	CORBA_unsigned_long retval;
+
+	if(poa->first_free_id) {
+		retval = poa->first_free_id;
+		poa->first_free_id = GPOINTER_TO_UINT(g_ptr_array_index(poa->objnum_to_obj,
+						       retval));
+	} else {
+		retval = poa->objnum_to_obj->len;
+		g_ptr_array_add(poa->objnum_to_obj,
+				obj);
+	}
+
+	return retval;
+}
+
+static CORBA_ORB
 get_orb_for_poa(PortableServer_POA poa)
 {
 	if(poa->orb)
@@ -699,6 +718,7 @@ PortableServer_POA_activate_object(PortableServer_POA obj,
 	new_obj->servant = p_servant;
 	ORBIT_OBJECT_KEY(servant->_private)->object = new_obj;
 	new_obj->orb = get_orb_for_poa(obj);
+	new_obj->objnum = get_objnum_for_obj(obj, new_obj);
 
 	g_hash_table_insert(obj->active_object_map,
 			    new_obj->object_id,
@@ -741,6 +761,7 @@ PortableServer_POA_activate_object_with_id(PortableServer_POA obj,
 	memcpy(newobj->object_id->_buffer, id->_buffer, id->_length);
 	newobj->poa = obj;
 	newobj->orb = get_orb_for_poa(obj);
+	newobj->objnum = get_objnum_for_obj(obj, newobj);
 
 	newobj->servant = p_servant;
 
@@ -776,6 +797,9 @@ PortableServer_POA_deactivate_object(PortableServer_POA obj,
 		return;
 	}
 
+	g_ptr_array_index(obj->objnum_to_obj, oldobj->objnum) = GUINT_TO_POINTER(obj->first_free_id);
+	obj->first_free_id = oldobj->objnum;
+
 	g_hash_table_remove(obj->active_object_map, oid);
 
 	if(obj->request_processing == PortableServer_USE_SERVANT_MANAGER) {
@@ -845,8 +869,15 @@ PortableServer_Servant PortableServer_POA_reference_to_servant(PortableServer_PO
 				    NULL);
 	}
 
-	objinfo = g_hash_table_lookup(obj->active_object_map,
-				      strrchr(reference->active_profile->object_key._buffer, '/') + 1);
+	{
+		CORBA_unsigned_long onum;
+		onum = 
+			*(CORBA_unsigned_long *)(reference->active_profile->object_key._buffer + sizeof(CORBA_unsigned_long));
+		if(onum < obj->objnum_to_obj->len)
+			objinfo = g_ptr_array_index(obj->objnum_to_obj, onum);
+		else
+			objinfo = NULL;
+	}
 
 	if(!objinfo) {
 		CORBA_exception_set(ev, CORBA_USER_EXCEPTION,
@@ -905,13 +936,9 @@ CORBA_Object PortableServer_POA_id_to_reference(PortableServer_POA obj,
 
 	pobj = g_hash_table_lookup(obj->active_object_map, oid);
 
-	if(pobj && pobj->servant) {
-		objkey = ORBIT_OBJECT_KEY(((PortableServer_ServantBase *)pobj->servant)->_private);
-		type_id=g_strdup(objkey->class_info->class_name);
-
-	} else {
-		type_id=g_strdup("Null");
-	}
+	g_assert(pobj);
+	objkey = ORBIT_OBJECT_KEY(((PortableServer_ServantBase *)pobj->servant)->_private);
+	type_id=g_strdup(objkey->class_info->class_name);
 
 	/* Do the local connection first, so it will be attempted first by
 	   the client parsing the IOR string
@@ -922,7 +949,7 @@ CORBA_Object PortableServer_POA_id_to_reference(PortableServer_POA obj,
 		object_info->profile_type=IOP_TAG_ORBIT_SPECIFIC;
 		object_info->iiop_major = 1;
 		object_info->iiop_minor = 0;
-		ORBit_POA_find_object_key_for_oid(obj, oid, &object_info->object_key);
+		ORBit_POA_find_object_key_for_oid(obj, pobj, &object_info->object_key);
 
 #ifdef HAVE_IPV6
 		if(orb->cnx.ipv6) {
@@ -944,7 +971,7 @@ CORBA_Object PortableServer_POA_id_to_reference(PortableServer_POA obj,
 		object_info->profile_type = IOP_TAG_INTERNET_IOP;
 		object_info->iiop_major = 1;
 		object_info->iiop_minor = 0;
-		ORBit_POA_find_object_key_for_oid(obj, oid, &object_info->object_key);
+		ORBit_POA_find_object_key_for_oid(obj, pobj, &object_info->object_key);
 
 		object_info->tag.iopinfo.host = g_strdup(IIOP_CONNECTION(orb->cnx.ipv4)->u.ipv4.hostname);
 		object_info->tag.iopinfo.port = ntohs(IIOP_CONNECTION(orb->cnx.ipv4)->u.ipv4.location.sin_port);
@@ -959,7 +986,7 @@ CORBA_Object PortableServer_POA_id_to_reference(PortableServer_POA obj,
 
 	if(retval != CORBA_OBJECT_NIL
 	   && ev->_major == CORBA_NO_EXCEPTION
-	   && objkey) {
+	   && objkey && objkey->class_info && objkey->class_info->init_local_objref) {
 		retval->vepv = g_new0(gpointer, ORBit_class_assignment_counter + 1);
 		objkey->class_info->init_local_objref(retval, pobj->servant);
 		retval->servant = pobj->servant;
@@ -1058,32 +1085,112 @@ void PortableServer_ServantBase__fini(PortableServer_Servant servant,
 
 CORBA_Identifier CORBA_ServerRequest_operation(CORBA_ServerRequest req, CORBA_Environment *env)
 {
-	g_assert(!"Not yet implemented");
-	return(NULL);
+	return CORBA_string_dup(req->rbuf->message.u.request.operation);
 }
 
-CORBA_Context *CORBA_ServerRequest_ctx(CORBA_ServerRequest req, CORBA_Environment *env)
+CORBA_Context
+CORBA_ServerRequest_ctx(CORBA_ServerRequest req, CORBA_Environment *env)
 {
-	g_assert(!"Not yet implemented");
-	return(NULL);
+	if(!req->params || req->did_ctx) {
+		CORBA_exception_set_system(env, ex_CORBA_BAD_INV_ORDER,
+					   CORBA_COMPLETED_NO);
+		return NULL;
+	}
+
+	return NULL;
 }
 
-void CORBA_ServerRequest_arguments(CORBA_ServerRequest req, CORBA_NVList *parameters, CORBA_Environment *env)
+void
+CORBA_ServerRequest_arguments(CORBA_ServerRequest req,
+			      CORBA_NVList *parameters,
+			      CORBA_Environment *env)
 {
-	g_assert(!"Not yet implemented");
-	return;
+	int i;
+
+	if(req->params) {
+		CORBA_exception_set_system(env, ex_CORBA_BAD_INV_ORDER,
+					   CORBA_COMPLETED_NO);
+		return;
+	}
+
+	req->params = parameters;
+
+	for(i = 0; i < parameters->list->len; i++) {
+		CORBA_NamedValue *cur;
+
+		cur = &g_array_index(parameters->list, CORBA_NamedValue, i);
+
+		if(cur->arg_modes & CORBA_ARG_OUT) continue;
+		cur->argument._value = ORBit_demarshal_arg(req->rbuf,
+							   cur->argument._type,
+							   TRUE,
+							   (CORBA_ORB)req->orb);
+		CORBA_any_set_release(&cur->argument, TRUE);
+	}
 }
 
-void CORBA_ServerRequest_set_result(CORBA_ServerRequest req, CORBA_any *value, CORBA_Environment *env)
+void
+CORBA_ServerRequest_set_result(CORBA_ServerRequest req,
+			       CORBA_any *value,
+			       CORBA_Environment *env)
 {
-	g_assert(!"Not yet implemented");
-	return;
+	if(req->sbuf) {
+		CORBA_exception_set_system(env, ex_CORBA_BAD_INV_ORDER,
+					   CORBA_COMPLETED_NO);
+		return;
+	}
+
+	req->sbuf = giop_send_reply_buffer_use(GIOP_MESSAGE_BUFFER(req->rbuf)->connection,
+					       NULL,
+					       req->rbuf->message.u.request.request_id,
+					       CORBA_NO_EXCEPTION);
+	if(!req->sbuf) {
+		CORBA_exception_set_system(env, ex_CORBA_COMM_FAILURE,
+					   CORBA_COMPLETED_NO);
+		return;
+	}
+		
+	ORBit_marshal_arg(req->sbuf, value->_value, value->_type);
 }
 
-void CORBA_ServerRequest_set_exception(CORBA_ServerRequest req, CORBA_exception_type major, CORBA_any *value, CORBA_Environment *env)
+void
+CORBA_ServerRequest_set_exception(CORBA_ServerRequest req,
+				  CORBA_exception_type major,
+				  CORBA_any *value,
+				  CORBA_Environment *env)
 {
-	g_assert(!"Not yet implemented");
-	return;
+	if(req->sbuf) {
+		CORBA_exception_set_system(env, ex_CORBA_BAD_INV_ORDER,
+					   CORBA_COMPLETED_NO);
+		return;
+	}
+
+	req->sbuf = giop_send_reply_buffer_use(GIOP_MESSAGE_BUFFER(req->rbuf)->connection,
+					       NULL,
+					       req->rbuf->message.u.request.request_id,
+					       major);
+	if(!req->sbuf) {
+		CORBA_exception_set_system(env, ex_CORBA_COMM_FAILURE,
+					   CORBA_COMPLETED_NO);
+		return;
+	}
+
+	req->did_exc = TRUE;
+
+	/* XXX do we really need to copy the repo_id into the
+	   send buffer? Or is there a way to assume that the CORBA_TypeCode
+	   value->_type will be around until after we send the message? */
+	{
+		CORBA_unsigned_long slen;
+		slen = strlen(value->_type->repo_id) + 1;
+		giop_send_buffer_append_mem_indirect_a(req->sbuf, &slen,
+						       sizeof(slen));
+		giop_send_buffer_append_mem_indirect(req->sbuf,
+						     value->_type->repo_id,
+						     slen);
+	}
+		
+	ORBit_marshal_arg(req->sbuf, value->_value, value->_type);
 }
 
 void
@@ -1127,3 +1234,78 @@ POA_PortableServer_ServantLocator__fini(PortableServer_Servant servant,
 {
 	PortableServer_ServantBase__fini(servant, ev);
 }
+
+/* POA-related DSI stuff */
+static void
+dynamic_impl_skel(PortableServer_DynamicImpl *_ORBIT_servant,
+		  GIOPRecvBuffer *_ORBIT_recv_buffer,
+		  CORBA_Environment *ev,
+		  PortableServer_DynamicImplRoutine invoke)
+{
+	/* here the magic occurs... */
+	struct CORBA_ServerRequest_type sr;
+
+	ORBit_pseudo_object_init(ORBIT_PSEUDO_OBJECT(&sr),
+				 ORBIT_PSEUDO_SERVERREQUEST, ev);
+
+	CORBA_Object_duplicate((CORBA_Object)&sr, ev); /* just to make
+							  sure it doesn't die
+							  elsewhere */
+
+	sr.rbuf = _ORBIT_recv_buffer;
+	sr.orb = GIOP_MESSAGE_BUFFER(_ORBIT_recv_buffer)->connection->orb_data;
+
+	_ORBIT_servant->vepv->PortableServer_DynamicImpl_epv->invoke(_ORBIT_servant,
+								     &sr);
+
+	if(sr.sbuf) {
+		int i;
+		for(i = 0; i < sr.params->list->len; i++) {
+			CORBA_NamedValue *cur;
+			
+			cur = &g_array_index(sr.params->list, CORBA_NamedValue, i);
+			
+			if(cur->arg_modes & CORBA_ARG_IN) continue;
+
+			ORBit_marshal_arg(sr.sbuf, cur->argument._value,
+					  cur->argument._type);
+		}
+
+		giop_send_buffer_write(sr.sbuf);
+		giop_send_buffer_unuse(sr.sbuf);
+	} else
+		g_warning("Yo, your DSI code is messed up! You forgot to set_result|set_exception");
+
+	CORBA_NVList_free(sr.params, ev);
+}
+
+static ORBitSkeleton 
+dynamic_impl_get_skel(PortableServer_DynamicImpl * servant,
+		      GIOPRecvBuffer * _ORBIT_recv_buffer,
+		      gpointer * impl)
+{
+	*impl = servant->vepv->PortableServer_DynamicImpl_epv->invoke;
+
+	return (ORBitSkeleton)dynamic_impl_skel;
+}
+
+void
+PortableServer_DynamicImpl__init(PortableServer_Servant servant,
+				 CORBA_Environment *ev)
+{
+	static const PortableServer_ClassInfo class_info =
+	{(gpointer) &dynamic_impl_get_skel, "IDL:CORBA/Object:1.0", NULL};
+	
+	PortableServer_ServantBase__init(servant, ev);
+	
+	ORBIT_OBJECT_KEY(((PortableServer_ServantBase *)servant)->_private)->class_info =
+		(PortableServer_ClassInfo *) & class_info;
+	
+}
+
+void PortableServer_DynamicImpl__fini(PortableServer_Servant servant,
+				      CORBA_Environment *ev)
+{
+	PortableServer_ServantBase__fini(servant, ev);
+}
+
