@@ -275,7 +275,7 @@ server_is_alive(CORBA_Object server)
   childpid = fork();
 
   if(!childpid) {
-    signal(SIGALRM, do_exit);
+    signal(SIGALRM, (void (*)(int))do_exit);
     alarm(1);
     cnx = _ORBit_object_get_connection(server);
     syslog(LOG_DEBUG, "server_is_alive: cnx[%s] = %p\n",
@@ -338,7 +338,7 @@ impl_CosNaming_NamingContext_bind(impl_POA_CosNaming_NamingContext * servant,
 				  CORBA_Object obj,
 				  CORBA_Environment * ev)
 {
-  RegisteredName *nom;
+  RegisteredName *nom = NULL;
 
   if(n->_length < 1) goto raise_exc;
 
@@ -384,7 +384,7 @@ impl_CosNaming_NamingContext_rebind(impl_POA_CosNaming_NamingContext * servant,
 				    CORBA_Object obj,
 				    CORBA_Environment * ev)
 {
-  RegisteredName *nom;
+  RegisteredName *nom = NULL;
 
   if(n->_length < 1) goto raise_exc;
 
@@ -424,7 +424,7 @@ impl_CosNaming_NamingContext_bind_context(impl_POA_CosNaming_NamingContext * ser
 					  CosNaming_NamingContext nc,
 					  CORBA_Environment * ev)
 {
-  RegisteredName *nom;
+  RegisteredName *nom = NULL;
 
   if(n->_length < 1) goto raise_exc;
 
@@ -470,7 +470,7 @@ impl_CosNaming_NamingContext_rebind_context(impl_POA_CosNaming_NamingContext * s
 					    CosNaming_NamingContext nc,
 					    CORBA_Environment * ev)
 {
-  RegisteredName *nom;
+  RegisteredName *nom = NULL;
 
   if(n->_length < 1) goto raise_exc;
   nom = g_hash_table_lookup(servant->names, n->_buffer);
@@ -508,7 +508,7 @@ impl_CosNaming_NamingContext_resolve(impl_POA_CosNaming_NamingContext * servant,
 				     CosNaming_Name * n,
 				     CORBA_Environment * ev)
 {
-  RegisteredName *nom;
+  RegisteredName *nom = NULL;
 
   if(n->_length < 1) goto raise_exc;
   nom = g_hash_table_lookup(servant->names, n->_buffer);
@@ -553,7 +553,7 @@ impl_CosNaming_NamingContext_unbind(impl_POA_CosNaming_NamingContext * servant,
 				    CosNaming_Name * n,
 				    CORBA_Environment * ev)
 {
-  RegisteredName *nom;
+  RegisteredName *nom = NULL;
 
   if(n->_length < 1) goto raise_exc;
   nom = g_hash_table_lookup(servant->names, n->_buffer);
@@ -592,7 +592,7 @@ impl_CosNaming_NamingContext_bind_new_context(impl_POA_CosNaming_NamingContext *
 					      CosNaming_Name * n,
 					      CORBA_Environment * ev)
 {
-  RegisteredName *nom;
+  RegisteredName *nom = NULL;
   CosNaming_NamingContext retval;
 
   if(n->_length < 1) goto raise_exc;
